# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities

__all__ = ['DirectoryRoleAssignmentArgs', 'DirectoryRoleAssignment']

@pulumi.input_type
class DirectoryRoleAssignmentArgs:
    def __init__(__self__, *,
                 principal_object_id: pulumi.Input[str],
                 role_id: pulumi.Input[str],
                 app_scope_id: Optional[pulumi.Input[str]] = None,
                 directory_scope_id: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a DirectoryRoleAssignment resource.
        :param pulumi.Input[str] principal_object_id: The object ID of the principal for you want to create a role assignment. Supported object types are Users, Groups or Service Principals. Changing this forces a new resource to be created.
        :param pulumi.Input[str] role_id: The template ID (in the case of built-in roles) or object ID (in the case of custom roles) of the directory role you want to assign. Changing this forces a new resource to be created.
        :param pulumi.Input[str] app_scope_id: Identifier of the app-specific scope when the assignment scope is app-specific. Cannot be used with `directory_scope_id`. See [official documentation](https://docs.microsoft.com/en-us/graph/api/rbacapplication-post-roleassignments?view=graph-rest-1.0&tabs=http) for example usage. Changing this forces a new resource to be created.
        :param pulumi.Input[str] directory_scope_id: Identifier of the directory object representing the scope of the assignment. Cannot be used with `app_scope_id`. See [official documentation](https://docs.microsoft.com/en-us/graph/api/rbacapplication-post-roleassignments?view=graph-rest-1.0&tabs=http) for example usage. Changing this forces a new resource to be created.
        """
        pulumi.set(__self__, "principal_object_id", principal_object_id)
        pulumi.set(__self__, "role_id", role_id)
        if app_scope_id is not None:
            pulumi.set(__self__, "app_scope_id", app_scope_id)
        if directory_scope_id is not None:
            pulumi.set(__self__, "directory_scope_id", directory_scope_id)

    @property
    @pulumi.getter(name="principalObjectId")
    def principal_object_id(self) -> pulumi.Input[str]:
        """
        The object ID of the principal for you want to create a role assignment. Supported object types are Users, Groups or Service Principals. Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "principal_object_id")

    @principal_object_id.setter
    def principal_object_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "principal_object_id", value)

    @property
    @pulumi.getter(name="roleId")
    def role_id(self) -> pulumi.Input[str]:
        """
        The template ID (in the case of built-in roles) or object ID (in the case of custom roles) of the directory role you want to assign. Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "role_id")

    @role_id.setter
    def role_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "role_id", value)

    @property
    @pulumi.getter(name="appScopeId")
    def app_scope_id(self) -> Optional[pulumi.Input[str]]:
        """
        Identifier of the app-specific scope when the assignment scope is app-specific. Cannot be used with `directory_scope_id`. See [official documentation](https://docs.microsoft.com/en-us/graph/api/rbacapplication-post-roleassignments?view=graph-rest-1.0&tabs=http) for example usage. Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "app_scope_id")

    @app_scope_id.setter
    def app_scope_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "app_scope_id", value)

    @property
    @pulumi.getter(name="directoryScopeId")
    def directory_scope_id(self) -> Optional[pulumi.Input[str]]:
        """
        Identifier of the directory object representing the scope of the assignment. Cannot be used with `app_scope_id`. See [official documentation](https://docs.microsoft.com/en-us/graph/api/rbacapplication-post-roleassignments?view=graph-rest-1.0&tabs=http) for example usage. Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "directory_scope_id")

    @directory_scope_id.setter
    def directory_scope_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "directory_scope_id", value)


@pulumi.input_type
class _DirectoryRoleAssignmentState:
    def __init__(__self__, *,
                 app_scope_id: Optional[pulumi.Input[str]] = None,
                 directory_scope_id: Optional[pulumi.Input[str]] = None,
                 principal_object_id: Optional[pulumi.Input[str]] = None,
                 role_id: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering DirectoryRoleAssignment resources.
        :param pulumi.Input[str] app_scope_id: Identifier of the app-specific scope when the assignment scope is app-specific. Cannot be used with `directory_scope_id`. See [official documentation](https://docs.microsoft.com/en-us/graph/api/rbacapplication-post-roleassignments?view=graph-rest-1.0&tabs=http) for example usage. Changing this forces a new resource to be created.
        :param pulumi.Input[str] directory_scope_id: Identifier of the directory object representing the scope of the assignment. Cannot be used with `app_scope_id`. See [official documentation](https://docs.microsoft.com/en-us/graph/api/rbacapplication-post-roleassignments?view=graph-rest-1.0&tabs=http) for example usage. Changing this forces a new resource to be created.
        :param pulumi.Input[str] principal_object_id: The object ID of the principal for you want to create a role assignment. Supported object types are Users, Groups or Service Principals. Changing this forces a new resource to be created.
        :param pulumi.Input[str] role_id: The template ID (in the case of built-in roles) or object ID (in the case of custom roles) of the directory role you want to assign. Changing this forces a new resource to be created.
        """
        if app_scope_id is not None:
            pulumi.set(__self__, "app_scope_id", app_scope_id)
        if directory_scope_id is not None:
            pulumi.set(__self__, "directory_scope_id", directory_scope_id)
        if principal_object_id is not None:
            pulumi.set(__self__, "principal_object_id", principal_object_id)
        if role_id is not None:
            pulumi.set(__self__, "role_id", role_id)

    @property
    @pulumi.getter(name="appScopeId")
    def app_scope_id(self) -> Optional[pulumi.Input[str]]:
        """
        Identifier of the app-specific scope when the assignment scope is app-specific. Cannot be used with `directory_scope_id`. See [official documentation](https://docs.microsoft.com/en-us/graph/api/rbacapplication-post-roleassignments?view=graph-rest-1.0&tabs=http) for example usage. Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "app_scope_id")

    @app_scope_id.setter
    def app_scope_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "app_scope_id", value)

    @property
    @pulumi.getter(name="directoryScopeId")
    def directory_scope_id(self) -> Optional[pulumi.Input[str]]:
        """
        Identifier of the directory object representing the scope of the assignment. Cannot be used with `app_scope_id`. See [official documentation](https://docs.microsoft.com/en-us/graph/api/rbacapplication-post-roleassignments?view=graph-rest-1.0&tabs=http) for example usage. Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "directory_scope_id")

    @directory_scope_id.setter
    def directory_scope_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "directory_scope_id", value)

    @property
    @pulumi.getter(name="principalObjectId")
    def principal_object_id(self) -> Optional[pulumi.Input[str]]:
        """
        The object ID of the principal for you want to create a role assignment. Supported object types are Users, Groups or Service Principals. Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "principal_object_id")

    @principal_object_id.setter
    def principal_object_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "principal_object_id", value)

    @property
    @pulumi.getter(name="roleId")
    def role_id(self) -> Optional[pulumi.Input[str]]:
        """
        The template ID (in the case of built-in roles) or object ID (in the case of custom roles) of the directory role you want to assign. Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "role_id")

    @role_id.setter
    def role_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "role_id", value)


class DirectoryRoleAssignment(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 app_scope_id: Optional[pulumi.Input[str]] = None,
                 directory_scope_id: Optional[pulumi.Input[str]] = None,
                 principal_object_id: Optional[pulumi.Input[str]] = None,
                 role_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Manages a single directory role assignment within Azure Active Directory.

        ## API Permissions

        The following API permissions are required in order to use this resource.

        When authenticated with a service principal, this resource requires one of the following application roles: `RoleManagement.ReadWrite.Directory` or `Directory.ReadWrite.All`

        When authenticated with a user principal, this resource requires one of the following directory roles: `Privileged Role Administrator` or `Global Administrator`

        ## Example Usage

        *Assignment for a built-in role*

        ```python
        import pulumi
        import pulumi_azuread as azuread

        example = azuread.get_user(user_principal_name="jdoe@example.com")
        example_directory_role = azuread.DirectoryRole("example", display_name="Security administrator")
        example_directory_role_assignment = azuread.DirectoryRoleAssignment("example",
            role_id=example_directory_role.template_id,
            principal_object_id=example.object_id)
        ```

        > Note the use of the `template_id` attribute when referencing built-in roles.

        *Assignment for a custom role*

        ```python
        import pulumi
        import pulumi_azuread as azuread

        example = azuread.get_user(user_principal_name="jdoe@example.com")
        example_custom_directory_role = azuread.CustomDirectoryRole("example",
            display_name="My Custom Role",
            enabled=True,
            version="1.0",
            permissions=[{
                "allowed_resource_actions": [
                    "microsoft.directory/applications/basic/update",
                    "microsoft.directory/applications/standard/read",
                ],
            }])
        example_directory_role_assignment = azuread.DirectoryRoleAssignment("example",
            role_id=example_custom_directory_role.object_id,
            principal_object_id=example.object_id)
        ```

        *Scoped assignment for an application*

        ## Import

        Directory role assignments can be imported using the ID of the assignment, e.g.

        ```sh
        $ pulumi import azuread:index/directoryRoleAssignment:DirectoryRoleAssignment example ePROZI_iKE653D_d6aoLHyr-lKgHI8ZGiIdz8CLVcng-1
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] app_scope_id: Identifier of the app-specific scope when the assignment scope is app-specific. Cannot be used with `directory_scope_id`. See [official documentation](https://docs.microsoft.com/en-us/graph/api/rbacapplication-post-roleassignments?view=graph-rest-1.0&tabs=http) for example usage. Changing this forces a new resource to be created.
        :param pulumi.Input[str] directory_scope_id: Identifier of the directory object representing the scope of the assignment. Cannot be used with `app_scope_id`. See [official documentation](https://docs.microsoft.com/en-us/graph/api/rbacapplication-post-roleassignments?view=graph-rest-1.0&tabs=http) for example usage. Changing this forces a new resource to be created.
        :param pulumi.Input[str] principal_object_id: The object ID of the principal for you want to create a role assignment. Supported object types are Users, Groups or Service Principals. Changing this forces a new resource to be created.
        :param pulumi.Input[str] role_id: The template ID (in the case of built-in roles) or object ID (in the case of custom roles) of the directory role you want to assign. Changing this forces a new resource to be created.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: DirectoryRoleAssignmentArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Manages a single directory role assignment within Azure Active Directory.

        ## API Permissions

        The following API permissions are required in order to use this resource.

        When authenticated with a service principal, this resource requires one of the following application roles: `RoleManagement.ReadWrite.Directory` or `Directory.ReadWrite.All`

        When authenticated with a user principal, this resource requires one of the following directory roles: `Privileged Role Administrator` or `Global Administrator`

        ## Example Usage

        *Assignment for a built-in role*

        ```python
        import pulumi
        import pulumi_azuread as azuread

        example = azuread.get_user(user_principal_name="jdoe@example.com")
        example_directory_role = azuread.DirectoryRole("example", display_name="Security administrator")
        example_directory_role_assignment = azuread.DirectoryRoleAssignment("example",
            role_id=example_directory_role.template_id,
            principal_object_id=example.object_id)
        ```

        > Note the use of the `template_id` attribute when referencing built-in roles.

        *Assignment for a custom role*

        ```python
        import pulumi
        import pulumi_azuread as azuread

        example = azuread.get_user(user_principal_name="jdoe@example.com")
        example_custom_directory_role = azuread.CustomDirectoryRole("example",
            display_name="My Custom Role",
            enabled=True,
            version="1.0",
            permissions=[{
                "allowed_resource_actions": [
                    "microsoft.directory/applications/basic/update",
                    "microsoft.directory/applications/standard/read",
                ],
            }])
        example_directory_role_assignment = azuread.DirectoryRoleAssignment("example",
            role_id=example_custom_directory_role.object_id,
            principal_object_id=example.object_id)
        ```

        *Scoped assignment for an application*

        ## Import

        Directory role assignments can be imported using the ID of the assignment, e.g.

        ```sh
        $ pulumi import azuread:index/directoryRoleAssignment:DirectoryRoleAssignment example ePROZI_iKE653D_d6aoLHyr-lKgHI8ZGiIdz8CLVcng-1
        ```

        :param str resource_name: The name of the resource.
        :param DirectoryRoleAssignmentArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(DirectoryRoleAssignmentArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 app_scope_id: Optional[pulumi.Input[str]] = None,
                 directory_scope_id: Optional[pulumi.Input[str]] = None,
                 principal_object_id: Optional[pulumi.Input[str]] = None,
                 role_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = DirectoryRoleAssignmentArgs.__new__(DirectoryRoleAssignmentArgs)

            __props__.__dict__["app_scope_id"] = app_scope_id
            __props__.__dict__["directory_scope_id"] = directory_scope_id
            if principal_object_id is None and not opts.urn:
                raise TypeError("Missing required property 'principal_object_id'")
            __props__.__dict__["principal_object_id"] = principal_object_id
            if role_id is None and not opts.urn:
                raise TypeError("Missing required property 'role_id'")
            __props__.__dict__["role_id"] = role_id
        super(DirectoryRoleAssignment, __self__).__init__(
            'azuread:index/directoryRoleAssignment:DirectoryRoleAssignment',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            app_scope_id: Optional[pulumi.Input[str]] = None,
            directory_scope_id: Optional[pulumi.Input[str]] = None,
            principal_object_id: Optional[pulumi.Input[str]] = None,
            role_id: Optional[pulumi.Input[str]] = None) -> 'DirectoryRoleAssignment':
        """
        Get an existing DirectoryRoleAssignment resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] app_scope_id: Identifier of the app-specific scope when the assignment scope is app-specific. Cannot be used with `directory_scope_id`. See [official documentation](https://docs.microsoft.com/en-us/graph/api/rbacapplication-post-roleassignments?view=graph-rest-1.0&tabs=http) for example usage. Changing this forces a new resource to be created.
        :param pulumi.Input[str] directory_scope_id: Identifier of the directory object representing the scope of the assignment. Cannot be used with `app_scope_id`. See [official documentation](https://docs.microsoft.com/en-us/graph/api/rbacapplication-post-roleassignments?view=graph-rest-1.0&tabs=http) for example usage. Changing this forces a new resource to be created.
        :param pulumi.Input[str] principal_object_id: The object ID of the principal for you want to create a role assignment. Supported object types are Users, Groups or Service Principals. Changing this forces a new resource to be created.
        :param pulumi.Input[str] role_id: The template ID (in the case of built-in roles) or object ID (in the case of custom roles) of the directory role you want to assign. Changing this forces a new resource to be created.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _DirectoryRoleAssignmentState.__new__(_DirectoryRoleAssignmentState)

        __props__.__dict__["app_scope_id"] = app_scope_id
        __props__.__dict__["directory_scope_id"] = directory_scope_id
        __props__.__dict__["principal_object_id"] = principal_object_id
        __props__.__dict__["role_id"] = role_id
        return DirectoryRoleAssignment(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="appScopeId")
    def app_scope_id(self) -> pulumi.Output[str]:
        """
        Identifier of the app-specific scope when the assignment scope is app-specific. Cannot be used with `directory_scope_id`. See [official documentation](https://docs.microsoft.com/en-us/graph/api/rbacapplication-post-roleassignments?view=graph-rest-1.0&tabs=http) for example usage. Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "app_scope_id")

    @property
    @pulumi.getter(name="directoryScopeId")
    def directory_scope_id(self) -> pulumi.Output[str]:
        """
        Identifier of the directory object representing the scope of the assignment. Cannot be used with `app_scope_id`. See [official documentation](https://docs.microsoft.com/en-us/graph/api/rbacapplication-post-roleassignments?view=graph-rest-1.0&tabs=http) for example usage. Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "directory_scope_id")

    @property
    @pulumi.getter(name="principalObjectId")
    def principal_object_id(self) -> pulumi.Output[str]:
        """
        The object ID of the principal for you want to create a role assignment. Supported object types are Users, Groups or Service Principals. Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "principal_object_id")

    @property
    @pulumi.getter(name="roleId")
    def role_id(self) -> pulumi.Output[str]:
        """
        The template ID (in the case of built-in roles) or object ID (in the case of custom roles) of the directory role you want to assign. Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "role_id")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities

__all__ = ['ApplicationRegistrationArgs', 'ApplicationRegistration']

@pulumi.input_type
class ApplicationRegistrationArgs:
    def __init__(__self__, *,
                 display_name: pulumi.Input[str],
                 description: Optional[pulumi.Input[str]] = None,
                 group_membership_claims: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 homepage_url: Optional[pulumi.Input[str]] = None,
                 implicit_access_token_issuance_enabled: Optional[pulumi.Input[bool]] = None,
                 implicit_id_token_issuance_enabled: Optional[pulumi.Input[bool]] = None,
                 logout_url: Optional[pulumi.Input[str]] = None,
                 marketing_url: Optional[pulumi.Input[str]] = None,
                 notes: Optional[pulumi.Input[str]] = None,
                 privacy_statement_url: Optional[pulumi.Input[str]] = None,
                 requested_access_token_version: Optional[pulumi.Input[int]] = None,
                 service_management_reference: Optional[pulumi.Input[str]] = None,
                 sign_in_audience: Optional[pulumi.Input[str]] = None,
                 support_url: Optional[pulumi.Input[str]] = None,
                 terms_of_service_url: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a ApplicationRegistration resource.
        :param pulumi.Input[str] display_name: The display name for the application.
        :param pulumi.Input[str] description: A description of the application, as shown to end users.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] group_membership_claims: Configures the `groups` claim issued in a user or OAuth access token that the app expects. Possible values are `None`, `SecurityGroup`, `DirectoryRole`, `ApplicationGroup` or `All`.
        :param pulumi.Input[str] homepage_url: Home page or landing page of the application.
        :param pulumi.Input[bool] implicit_access_token_issuance_enabled: Whether this web application can request an access token using OAuth implicit flow.
        :param pulumi.Input[bool] implicit_id_token_issuance_enabled: Whether this web application can request an ID token using OAuth implicit flow.
        :param pulumi.Input[str] logout_url: The URL that will be used by Microsoft's authorization service to sign out a user using front-channel, back-channel or SAML logout protocols.
        :param pulumi.Input[str] marketing_url: URL of the marketing page for the application.
        :param pulumi.Input[str] notes: User-specified notes relevant for the management of the application.
        :param pulumi.Input[str] privacy_statement_url: URL of the privacy statement for the application.
        :param pulumi.Input[int] requested_access_token_version: The access token version expected by this resource. Must be one of `1` or `2`, and must be `2` when `sign_in_audience` is either `AzureADandPersonalMicrosoftAccount` or `PersonalMicrosoftAccount` Defaults to `2`.
        :param pulumi.Input[str] service_management_reference: References application context information from a Service or Asset Management database.
        :param pulumi.Input[str] sign_in_audience: The Microsoft account types that are supported for the current application. Must be one of `AzureADMyOrg`, `AzureADMultipleOrgs`, `AzureADandPersonalMicrosoftAccount` or `PersonalMicrosoftAccount`. Defaults to `AzureADMyOrg`.
        :param pulumi.Input[str] support_url: URL of the support page for the application.
        :param pulumi.Input[str] terms_of_service_url: URL of the terms of service statement for the application.
        """
        pulumi.set(__self__, "display_name", display_name)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if group_membership_claims is not None:
            pulumi.set(__self__, "group_membership_claims", group_membership_claims)
        if homepage_url is not None:
            pulumi.set(__self__, "homepage_url", homepage_url)
        if implicit_access_token_issuance_enabled is not None:
            pulumi.set(__self__, "implicit_access_token_issuance_enabled", implicit_access_token_issuance_enabled)
        if implicit_id_token_issuance_enabled is not None:
            pulumi.set(__self__, "implicit_id_token_issuance_enabled", implicit_id_token_issuance_enabled)
        if logout_url is not None:
            pulumi.set(__self__, "logout_url", logout_url)
        if marketing_url is not None:
            pulumi.set(__self__, "marketing_url", marketing_url)
        if notes is not None:
            pulumi.set(__self__, "notes", notes)
        if privacy_statement_url is not None:
            pulumi.set(__self__, "privacy_statement_url", privacy_statement_url)
        if requested_access_token_version is not None:
            pulumi.set(__self__, "requested_access_token_version", requested_access_token_version)
        if service_management_reference is not None:
            pulumi.set(__self__, "service_management_reference", service_management_reference)
        if sign_in_audience is not None:
            pulumi.set(__self__, "sign_in_audience", sign_in_audience)
        if support_url is not None:
            pulumi.set(__self__, "support_url", support_url)
        if terms_of_service_url is not None:
            pulumi.set(__self__, "terms_of_service_url", terms_of_service_url)

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> pulumi.Input[str]:
        """
        The display name for the application.
        """
        return pulumi.get(self, "display_name")

    @display_name.setter
    def display_name(self, value: pulumi.Input[str]):
        pulumi.set(self, "display_name", value)

    @property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[str]]:
        """
        A description of the application, as shown to end users.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter(name="groupMembershipClaims")
    def group_membership_claims(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        Configures the `groups` claim issued in a user or OAuth access token that the app expects. Possible values are `None`, `SecurityGroup`, `DirectoryRole`, `ApplicationGroup` or `All`.
        """
        return pulumi.get(self, "group_membership_claims")

    @group_membership_claims.setter
    def group_membership_claims(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "group_membership_claims", value)

    @property
    @pulumi.getter(name="homepageUrl")
    def homepage_url(self) -> Optional[pulumi.Input[str]]:
        """
        Home page or landing page of the application.
        """
        return pulumi.get(self, "homepage_url")

    @homepage_url.setter
    def homepage_url(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "homepage_url", value)

    @property
    @pulumi.getter(name="implicitAccessTokenIssuanceEnabled")
    def implicit_access_token_issuance_enabled(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether this web application can request an access token using OAuth implicit flow.
        """
        return pulumi.get(self, "implicit_access_token_issuance_enabled")

    @implicit_access_token_issuance_enabled.setter
    def implicit_access_token_issuance_enabled(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "implicit_access_token_issuance_enabled", value)

    @property
    @pulumi.getter(name="implicitIdTokenIssuanceEnabled")
    def implicit_id_token_issuance_enabled(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether this web application can request an ID token using OAuth implicit flow.
        """
        return pulumi.get(self, "implicit_id_token_issuance_enabled")

    @implicit_id_token_issuance_enabled.setter
    def implicit_id_token_issuance_enabled(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "implicit_id_token_issuance_enabled", value)

    @property
    @pulumi.getter(name="logoutUrl")
    def logout_url(self) -> Optional[pulumi.Input[str]]:
        """
        The URL that will be used by Microsoft's authorization service to sign out a user using front-channel, back-channel or SAML logout protocols.
        """
        return pulumi.get(self, "logout_url")

    @logout_url.setter
    def logout_url(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "logout_url", value)

    @property
    @pulumi.getter(name="marketingUrl")
    def marketing_url(self) -> Optional[pulumi.Input[str]]:
        """
        URL of the marketing page for the application.
        """
        return pulumi.get(self, "marketing_url")

    @marketing_url.setter
    def marketing_url(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "marketing_url", value)

    @property
    @pulumi.getter
    def notes(self) -> Optional[pulumi.Input[str]]:
        """
        User-specified notes relevant for the management of the application.
        """
        return pulumi.get(self, "notes")

    @notes.setter
    def notes(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "notes", value)

    @property
    @pulumi.getter(name="privacyStatementUrl")
    def privacy_statement_url(self) -> Optional[pulumi.Input[str]]:
        """
        URL of the privacy statement for the application.
        """
        return pulumi.get(self, "privacy_statement_url")

    @privacy_statement_url.setter
    def privacy_statement_url(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "privacy_statement_url", value)

    @property
    @pulumi.getter(name="requestedAccessTokenVersion")
    def requested_access_token_version(self) -> Optional[pulumi.Input[int]]:
        """
        The access token version expected by this resource. Must be one of `1` or `2`, and must be `2` when `sign_in_audience` is either `AzureADandPersonalMicrosoftAccount` or `PersonalMicrosoftAccount` Defaults to `2`.
        """
        return pulumi.get(self, "requested_access_token_version")

    @requested_access_token_version.setter
    def requested_access_token_version(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "requested_access_token_version", value)

    @property
    @pulumi.getter(name="serviceManagementReference")
    def service_management_reference(self) -> Optional[pulumi.Input[str]]:
        """
        References application context information from a Service or Asset Management database.
        """
        return pulumi.get(self, "service_management_reference")

    @service_management_reference.setter
    def service_management_reference(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "service_management_reference", value)

    @property
    @pulumi.getter(name="signInAudience")
    def sign_in_audience(self) -> Optional[pulumi.Input[str]]:
        """
        The Microsoft account types that are supported for the current application. Must be one of `AzureADMyOrg`, `AzureADMultipleOrgs`, `AzureADandPersonalMicrosoftAccount` or `PersonalMicrosoftAccount`. Defaults to `AzureADMyOrg`.
        """
        return pulumi.get(self, "sign_in_audience")

    @sign_in_audience.setter
    def sign_in_audience(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "sign_in_audience", value)

    @property
    @pulumi.getter(name="supportUrl")
    def support_url(self) -> Optional[pulumi.Input[str]]:
        """
        URL of the support page for the application.
        """
        return pulumi.get(self, "support_url")

    @support_url.setter
    def support_url(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "support_url", value)

    @property
    @pulumi.getter(name="termsOfServiceUrl")
    def terms_of_service_url(self) -> Optional[pulumi.Input[str]]:
        """
        URL of the terms of service statement for the application.
        """
        return pulumi.get(self, "terms_of_service_url")

    @terms_of_service_url.setter
    def terms_of_service_url(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "terms_of_service_url", value)


@pulumi.input_type
class _ApplicationRegistrationState:
    def __init__(__self__, *,
                 client_id: Optional[pulumi.Input[str]] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 disabled_by_microsoft: Optional[pulumi.Input[str]] = None,
                 display_name: Optional[pulumi.Input[str]] = None,
                 group_membership_claims: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 homepage_url: Optional[pulumi.Input[str]] = None,
                 implicit_access_token_issuance_enabled: Optional[pulumi.Input[bool]] = None,
                 implicit_id_token_issuance_enabled: Optional[pulumi.Input[bool]] = None,
                 logout_url: Optional[pulumi.Input[str]] = None,
                 marketing_url: Optional[pulumi.Input[str]] = None,
                 notes: Optional[pulumi.Input[str]] = None,
                 object_id: Optional[pulumi.Input[str]] = None,
                 privacy_statement_url: Optional[pulumi.Input[str]] = None,
                 publisher_domain: Optional[pulumi.Input[str]] = None,
                 requested_access_token_version: Optional[pulumi.Input[int]] = None,
                 service_management_reference: Optional[pulumi.Input[str]] = None,
                 sign_in_audience: Optional[pulumi.Input[str]] = None,
                 support_url: Optional[pulumi.Input[str]] = None,
                 terms_of_service_url: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering ApplicationRegistration resources.
        :param pulumi.Input[str] client_id: The Client ID for the application, which is globally unique.
        :param pulumi.Input[str] description: A description of the application, as shown to end users.
        :param pulumi.Input[str] disabled_by_microsoft: Whether Microsoft has disabled the registered application. If the application is disabled, this will be a string indicating the status/reason, e.g. `DisabledDueToViolationOfServicesAgreement`
        :param pulumi.Input[str] display_name: The display name for the application.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] group_membership_claims: Configures the `groups` claim issued in a user or OAuth access token that the app expects. Possible values are `None`, `SecurityGroup`, `DirectoryRole`, `ApplicationGroup` or `All`.
        :param pulumi.Input[str] homepage_url: Home page or landing page of the application.
        :param pulumi.Input[bool] implicit_access_token_issuance_enabled: Whether this web application can request an access token using OAuth implicit flow.
        :param pulumi.Input[bool] implicit_id_token_issuance_enabled: Whether this web application can request an ID token using OAuth implicit flow.
        :param pulumi.Input[str] logout_url: The URL that will be used by Microsoft's authorization service to sign out a user using front-channel, back-channel or SAML logout protocols.
        :param pulumi.Input[str] marketing_url: URL of the marketing page for the application.
        :param pulumi.Input[str] notes: User-specified notes relevant for the management of the application.
        :param pulumi.Input[str] object_id: The object ID of the application within the tenant.
        :param pulumi.Input[str] privacy_statement_url: URL of the privacy statement for the application.
        :param pulumi.Input[str] publisher_domain: The verified publisher domain for the application.
        :param pulumi.Input[int] requested_access_token_version: The access token version expected by this resource. Must be one of `1` or `2`, and must be `2` when `sign_in_audience` is either `AzureADandPersonalMicrosoftAccount` or `PersonalMicrosoftAccount` Defaults to `2`.
        :param pulumi.Input[str] service_management_reference: References application context information from a Service or Asset Management database.
        :param pulumi.Input[str] sign_in_audience: The Microsoft account types that are supported for the current application. Must be one of `AzureADMyOrg`, `AzureADMultipleOrgs`, `AzureADandPersonalMicrosoftAccount` or `PersonalMicrosoftAccount`. Defaults to `AzureADMyOrg`.
        :param pulumi.Input[str] support_url: URL of the support page for the application.
        :param pulumi.Input[str] terms_of_service_url: URL of the terms of service statement for the application.
        """
        if client_id is not None:
            pulumi.set(__self__, "client_id", client_id)
        if description is not None:
            pulumi.set(__self__, "description", description)
        if disabled_by_microsoft is not None:
            pulumi.set(__self__, "disabled_by_microsoft", disabled_by_microsoft)
        if display_name is not None:
            pulumi.set(__self__, "display_name", display_name)
        if group_membership_claims is not None:
            pulumi.set(__self__, "group_membership_claims", group_membership_claims)
        if homepage_url is not None:
            pulumi.set(__self__, "homepage_url", homepage_url)
        if implicit_access_token_issuance_enabled is not None:
            pulumi.set(__self__, "implicit_access_token_issuance_enabled", implicit_access_token_issuance_enabled)
        if implicit_id_token_issuance_enabled is not None:
            pulumi.set(__self__, "implicit_id_token_issuance_enabled", implicit_id_token_issuance_enabled)
        if logout_url is not None:
            pulumi.set(__self__, "logout_url", logout_url)
        if marketing_url is not None:
            pulumi.set(__self__, "marketing_url", marketing_url)
        if notes is not None:
            pulumi.set(__self__, "notes", notes)
        if object_id is not None:
            pulumi.set(__self__, "object_id", object_id)
        if privacy_statement_url is not None:
            pulumi.set(__self__, "privacy_statement_url", privacy_statement_url)
        if publisher_domain is not None:
            pulumi.set(__self__, "publisher_domain", publisher_domain)
        if requested_access_token_version is not None:
            pulumi.set(__self__, "requested_access_token_version", requested_access_token_version)
        if service_management_reference is not None:
            pulumi.set(__self__, "service_management_reference", service_management_reference)
        if sign_in_audience is not None:
            pulumi.set(__self__, "sign_in_audience", sign_in_audience)
        if support_url is not None:
            pulumi.set(__self__, "support_url", support_url)
        if terms_of_service_url is not None:
            pulumi.set(__self__, "terms_of_service_url", terms_of_service_url)

    @property
    @pulumi.getter(name="clientId")
    def client_id(self) -> Optional[pulumi.Input[str]]:
        """
        The Client ID for the application, which is globally unique.
        """
        return pulumi.get(self, "client_id")

    @client_id.setter
    def client_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "client_id", value)

    @property
    @pulumi.getter
    def description(self) -> Optional[pulumi.Input[str]]:
        """
        A description of the application, as shown to end users.
        """
        return pulumi.get(self, "description")

    @description.setter
    def description(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "description", value)

    @property
    @pulumi.getter(name="disabledByMicrosoft")
    def disabled_by_microsoft(self) -> Optional[pulumi.Input[str]]:
        """
        Whether Microsoft has disabled the registered application. If the application is disabled, this will be a string indicating the status/reason, e.g. `DisabledDueToViolationOfServicesAgreement`
        """
        return pulumi.get(self, "disabled_by_microsoft")

    @disabled_by_microsoft.setter
    def disabled_by_microsoft(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "disabled_by_microsoft", value)

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> Optional[pulumi.Input[str]]:
        """
        The display name for the application.
        """
        return pulumi.get(self, "display_name")

    @display_name.setter
    def display_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "display_name", value)

    @property
    @pulumi.getter(name="groupMembershipClaims")
    def group_membership_claims(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        Configures the `groups` claim issued in a user or OAuth access token that the app expects. Possible values are `None`, `SecurityGroup`, `DirectoryRole`, `ApplicationGroup` or `All`.
        """
        return pulumi.get(self, "group_membership_claims")

    @group_membership_claims.setter
    def group_membership_claims(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "group_membership_claims", value)

    @property
    @pulumi.getter(name="homepageUrl")
    def homepage_url(self) -> Optional[pulumi.Input[str]]:
        """
        Home page or landing page of the application.
        """
        return pulumi.get(self, "homepage_url")

    @homepage_url.setter
    def homepage_url(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "homepage_url", value)

    @property
    @pulumi.getter(name="implicitAccessTokenIssuanceEnabled")
    def implicit_access_token_issuance_enabled(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether this web application can request an access token using OAuth implicit flow.
        """
        return pulumi.get(self, "implicit_access_token_issuance_enabled")

    @implicit_access_token_issuance_enabled.setter
    def implicit_access_token_issuance_enabled(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "implicit_access_token_issuance_enabled", value)

    @property
    @pulumi.getter(name="implicitIdTokenIssuanceEnabled")
    def implicit_id_token_issuance_enabled(self) -> Optional[pulumi.Input[bool]]:
        """
        Whether this web application can request an ID token using OAuth implicit flow.
        """
        return pulumi.get(self, "implicit_id_token_issuance_enabled")

    @implicit_id_token_issuance_enabled.setter
    def implicit_id_token_issuance_enabled(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "implicit_id_token_issuance_enabled", value)

    @property
    @pulumi.getter(name="logoutUrl")
    def logout_url(self) -> Optional[pulumi.Input[str]]:
        """
        The URL that will be used by Microsoft's authorization service to sign out a user using front-channel, back-channel or SAML logout protocols.
        """
        return pulumi.get(self, "logout_url")

    @logout_url.setter
    def logout_url(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "logout_url", value)

    @property
    @pulumi.getter(name="marketingUrl")
    def marketing_url(self) -> Optional[pulumi.Input[str]]:
        """
        URL of the marketing page for the application.
        """
        return pulumi.get(self, "marketing_url")

    @marketing_url.setter
    def marketing_url(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "marketing_url", value)

    @property
    @pulumi.getter
    def notes(self) -> Optional[pulumi.Input[str]]:
        """
        User-specified notes relevant for the management of the application.
        """
        return pulumi.get(self, "notes")

    @notes.setter
    def notes(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "notes", value)

    @property
    @pulumi.getter(name="objectId")
    def object_id(self) -> Optional[pulumi.Input[str]]:
        """
        The object ID of the application within the tenant.
        """
        return pulumi.get(self, "object_id")

    @object_id.setter
    def object_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "object_id", value)

    @property
    @pulumi.getter(name="privacyStatementUrl")
    def privacy_statement_url(self) -> Optional[pulumi.Input[str]]:
        """
        URL of the privacy statement for the application.
        """
        return pulumi.get(self, "privacy_statement_url")

    @privacy_statement_url.setter
    def privacy_statement_url(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "privacy_statement_url", value)

    @property
    @pulumi.getter(name="publisherDomain")
    def publisher_domain(self) -> Optional[pulumi.Input[str]]:
        """
        The verified publisher domain for the application.
        """
        return pulumi.get(self, "publisher_domain")

    @publisher_domain.setter
    def publisher_domain(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "publisher_domain", value)

    @property
    @pulumi.getter(name="requestedAccessTokenVersion")
    def requested_access_token_version(self) -> Optional[pulumi.Input[int]]:
        """
        The access token version expected by this resource. Must be one of `1` or `2`, and must be `2` when `sign_in_audience` is either `AzureADandPersonalMicrosoftAccount` or `PersonalMicrosoftAccount` Defaults to `2`.
        """
        return pulumi.get(self, "requested_access_token_version")

    @requested_access_token_version.setter
    def requested_access_token_version(self, value: Optional[pulumi.Input[int]]):
        pulumi.set(self, "requested_access_token_version", value)

    @property
    @pulumi.getter(name="serviceManagementReference")
    def service_management_reference(self) -> Optional[pulumi.Input[str]]:
        """
        References application context information from a Service or Asset Management database.
        """
        return pulumi.get(self, "service_management_reference")

    @service_management_reference.setter
    def service_management_reference(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "service_management_reference", value)

    @property
    @pulumi.getter(name="signInAudience")
    def sign_in_audience(self) -> Optional[pulumi.Input[str]]:
        """
        The Microsoft account types that are supported for the current application. Must be one of `AzureADMyOrg`, `AzureADMultipleOrgs`, `AzureADandPersonalMicrosoftAccount` or `PersonalMicrosoftAccount`. Defaults to `AzureADMyOrg`.
        """
        return pulumi.get(self, "sign_in_audience")

    @sign_in_audience.setter
    def sign_in_audience(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "sign_in_audience", value)

    @property
    @pulumi.getter(name="supportUrl")
    def support_url(self) -> Optional[pulumi.Input[str]]:
        """
        URL of the support page for the application.
        """
        return pulumi.get(self, "support_url")

    @support_url.setter
    def support_url(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "support_url", value)

    @property
    @pulumi.getter(name="termsOfServiceUrl")
    def terms_of_service_url(self) -> Optional[pulumi.Input[str]]:
        """
        URL of the terms of service statement for the application.
        """
        return pulumi.get(self, "terms_of_service_url")

    @terms_of_service_url.setter
    def terms_of_service_url(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "terms_of_service_url", value)


class ApplicationRegistration(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 display_name: Optional[pulumi.Input[str]] = None,
                 group_membership_claims: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 homepage_url: Optional[pulumi.Input[str]] = None,
                 implicit_access_token_issuance_enabled: Optional[pulumi.Input[bool]] = None,
                 implicit_id_token_issuance_enabled: Optional[pulumi.Input[bool]] = None,
                 logout_url: Optional[pulumi.Input[str]] = None,
                 marketing_url: Optional[pulumi.Input[str]] = None,
                 notes: Optional[pulumi.Input[str]] = None,
                 privacy_statement_url: Optional[pulumi.Input[str]] = None,
                 requested_access_token_version: Optional[pulumi.Input[int]] = None,
                 service_management_reference: Optional[pulumi.Input[str]] = None,
                 sign_in_audience: Optional[pulumi.Input[str]] = None,
                 support_url: Optional[pulumi.Input[str]] = None,
                 terms_of_service_url: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Manages an application registration within Azure Active Directory.

        For a more comprehensive alternative, please see the Application resource. Please note that this resource should not be used together with the `Application` resource when managing the same application.

        ## API Permissions

        The following API permissions are required in order to use this resource.

        When authenticated with a service principal, this resource requires one of the following application roles: `Application.ReadWrite.OwnedBy` or `Application.ReadWrite.All`

        When authenticated with a user principal, this resource may require one of the following directory roles: `Application Administrator` or `Global Administrator`

        ## Example Usage

        ```python
        import pulumi
        import pulumi_azuread as azuread

        example = azuread.ApplicationRegistration("example",
            display_name="Example Application",
            description="My example application",
            sign_in_audience="AzureADMyOrg",
            homepage_url="https://app.example.com/",
            logout_url="https://app.example.com/logout",
            marketing_url="https://example.com/",
            privacy_statement_url="https://example.com/privacy",
            support_url="https://support.example.com/",
            terms_of_service_url="https://example.com/terms")
        ```

        ## Import

        Application Registrations can be imported using the object ID of the application, in the following format.

        ```sh
        $ pulumi import azuread:index/applicationRegistration:ApplicationRegistration example /applications/00000000-0000-0000-0000-000000000000
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] description: A description of the application, as shown to end users.
        :param pulumi.Input[str] display_name: The display name for the application.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] group_membership_claims: Configures the `groups` claim issued in a user or OAuth access token that the app expects. Possible values are `None`, `SecurityGroup`, `DirectoryRole`, `ApplicationGroup` or `All`.
        :param pulumi.Input[str] homepage_url: Home page or landing page of the application.
        :param pulumi.Input[bool] implicit_access_token_issuance_enabled: Whether this web application can request an access token using OAuth implicit flow.
        :param pulumi.Input[bool] implicit_id_token_issuance_enabled: Whether this web application can request an ID token using OAuth implicit flow.
        :param pulumi.Input[str] logout_url: The URL that will be used by Microsoft's authorization service to sign out a user using front-channel, back-channel or SAML logout protocols.
        :param pulumi.Input[str] marketing_url: URL of the marketing page for the application.
        :param pulumi.Input[str] notes: User-specified notes relevant for the management of the application.
        :param pulumi.Input[str] privacy_statement_url: URL of the privacy statement for the application.
        :param pulumi.Input[int] requested_access_token_version: The access token version expected by this resource. Must be one of `1` or `2`, and must be `2` when `sign_in_audience` is either `AzureADandPersonalMicrosoftAccount` or `PersonalMicrosoftAccount` Defaults to `2`.
        :param pulumi.Input[str] service_management_reference: References application context information from a Service or Asset Management database.
        :param pulumi.Input[str] sign_in_audience: The Microsoft account types that are supported for the current application. Must be one of `AzureADMyOrg`, `AzureADMultipleOrgs`, `AzureADandPersonalMicrosoftAccount` or `PersonalMicrosoftAccount`. Defaults to `AzureADMyOrg`.
        :param pulumi.Input[str] support_url: URL of the support page for the application.
        :param pulumi.Input[str] terms_of_service_url: URL of the terms of service statement for the application.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: ApplicationRegistrationArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Manages an application registration within Azure Active Directory.

        For a more comprehensive alternative, please see the Application resource. Please note that this resource should not be used together with the `Application` resource when managing the same application.

        ## API Permissions

        The following API permissions are required in order to use this resource.

        When authenticated with a service principal, this resource requires one of the following application roles: `Application.ReadWrite.OwnedBy` or `Application.ReadWrite.All`

        When authenticated with a user principal, this resource may require one of the following directory roles: `Application Administrator` or `Global Administrator`

        ## Example Usage

        ```python
        import pulumi
        import pulumi_azuread as azuread

        example = azuread.ApplicationRegistration("example",
            display_name="Example Application",
            description="My example application",
            sign_in_audience="AzureADMyOrg",
            homepage_url="https://app.example.com/",
            logout_url="https://app.example.com/logout",
            marketing_url="https://example.com/",
            privacy_statement_url="https://example.com/privacy",
            support_url="https://support.example.com/",
            terms_of_service_url="https://example.com/terms")
        ```

        ## Import

        Application Registrations can be imported using the object ID of the application, in the following format.

        ```sh
        $ pulumi import azuread:index/applicationRegistration:ApplicationRegistration example /applications/00000000-0000-0000-0000-000000000000
        ```

        :param str resource_name: The name of the resource.
        :param ApplicationRegistrationArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ApplicationRegistrationArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 description: Optional[pulumi.Input[str]] = None,
                 display_name: Optional[pulumi.Input[str]] = None,
                 group_membership_claims: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 homepage_url: Optional[pulumi.Input[str]] = None,
                 implicit_access_token_issuance_enabled: Optional[pulumi.Input[bool]] = None,
                 implicit_id_token_issuance_enabled: Optional[pulumi.Input[bool]] = None,
                 logout_url: Optional[pulumi.Input[str]] = None,
                 marketing_url: Optional[pulumi.Input[str]] = None,
                 notes: Optional[pulumi.Input[str]] = None,
                 privacy_statement_url: Optional[pulumi.Input[str]] = None,
                 requested_access_token_version: Optional[pulumi.Input[int]] = None,
                 service_management_reference: Optional[pulumi.Input[str]] = None,
                 sign_in_audience: Optional[pulumi.Input[str]] = None,
                 support_url: Optional[pulumi.Input[str]] = None,
                 terms_of_service_url: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ApplicationRegistrationArgs.__new__(ApplicationRegistrationArgs)

            __props__.__dict__["description"] = description
            if display_name is None and not opts.urn:
                raise TypeError("Missing required property 'display_name'")
            __props__.__dict__["display_name"] = display_name
            __props__.__dict__["group_membership_claims"] = group_membership_claims
            __props__.__dict__["homepage_url"] = homepage_url
            __props__.__dict__["implicit_access_token_issuance_enabled"] = implicit_access_token_issuance_enabled
            __props__.__dict__["implicit_id_token_issuance_enabled"] = implicit_id_token_issuance_enabled
            __props__.__dict__["logout_url"] = logout_url
            __props__.__dict__["marketing_url"] = marketing_url
            __props__.__dict__["notes"] = notes
            __props__.__dict__["privacy_statement_url"] = privacy_statement_url
            __props__.__dict__["requested_access_token_version"] = requested_access_token_version
            __props__.__dict__["service_management_reference"] = service_management_reference
            __props__.__dict__["sign_in_audience"] = sign_in_audience
            __props__.__dict__["support_url"] = support_url
            __props__.__dict__["terms_of_service_url"] = terms_of_service_url
            __props__.__dict__["client_id"] = None
            __props__.__dict__["disabled_by_microsoft"] = None
            __props__.__dict__["object_id"] = None
            __props__.__dict__["publisher_domain"] = None
        super(ApplicationRegistration, __self__).__init__(
            'azuread:index/applicationRegistration:ApplicationRegistration',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            client_id: Optional[pulumi.Input[str]] = None,
            description: Optional[pulumi.Input[str]] = None,
            disabled_by_microsoft: Optional[pulumi.Input[str]] = None,
            display_name: Optional[pulumi.Input[str]] = None,
            group_membership_claims: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
            homepage_url: Optional[pulumi.Input[str]] = None,
            implicit_access_token_issuance_enabled: Optional[pulumi.Input[bool]] = None,
            implicit_id_token_issuance_enabled: Optional[pulumi.Input[bool]] = None,
            logout_url: Optional[pulumi.Input[str]] = None,
            marketing_url: Optional[pulumi.Input[str]] = None,
            notes: Optional[pulumi.Input[str]] = None,
            object_id: Optional[pulumi.Input[str]] = None,
            privacy_statement_url: Optional[pulumi.Input[str]] = None,
            publisher_domain: Optional[pulumi.Input[str]] = None,
            requested_access_token_version: Optional[pulumi.Input[int]] = None,
            service_management_reference: Optional[pulumi.Input[str]] = None,
            sign_in_audience: Optional[pulumi.Input[str]] = None,
            support_url: Optional[pulumi.Input[str]] = None,
            terms_of_service_url: Optional[pulumi.Input[str]] = None) -> 'ApplicationRegistration':
        """
        Get an existing ApplicationRegistration resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] client_id: The Client ID for the application, which is globally unique.
        :param pulumi.Input[str] description: A description of the application, as shown to end users.
        :param pulumi.Input[str] disabled_by_microsoft: Whether Microsoft has disabled the registered application. If the application is disabled, this will be a string indicating the status/reason, e.g. `DisabledDueToViolationOfServicesAgreement`
        :param pulumi.Input[str] display_name: The display name for the application.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] group_membership_claims: Configures the `groups` claim issued in a user or OAuth access token that the app expects. Possible values are `None`, `SecurityGroup`, `DirectoryRole`, `ApplicationGroup` or `All`.
        :param pulumi.Input[str] homepage_url: Home page or landing page of the application.
        :param pulumi.Input[bool] implicit_access_token_issuance_enabled: Whether this web application can request an access token using OAuth implicit flow.
        :param pulumi.Input[bool] implicit_id_token_issuance_enabled: Whether this web application can request an ID token using OAuth implicit flow.
        :param pulumi.Input[str] logout_url: The URL that will be used by Microsoft's authorization service to sign out a user using front-channel, back-channel or SAML logout protocols.
        :param pulumi.Input[str] marketing_url: URL of the marketing page for the application.
        :param pulumi.Input[str] notes: User-specified notes relevant for the management of the application.
        :param pulumi.Input[str] object_id: The object ID of the application within the tenant.
        :param pulumi.Input[str] privacy_statement_url: URL of the privacy statement for the application.
        :param pulumi.Input[str] publisher_domain: The verified publisher domain for the application.
        :param pulumi.Input[int] requested_access_token_version: The access token version expected by this resource. Must be one of `1` or `2`, and must be `2` when `sign_in_audience` is either `AzureADandPersonalMicrosoftAccount` or `PersonalMicrosoftAccount` Defaults to `2`.
        :param pulumi.Input[str] service_management_reference: References application context information from a Service or Asset Management database.
        :param pulumi.Input[str] sign_in_audience: The Microsoft account types that are supported for the current application. Must be one of `AzureADMyOrg`, `AzureADMultipleOrgs`, `AzureADandPersonalMicrosoftAccount` or `PersonalMicrosoftAccount`. Defaults to `AzureADMyOrg`.
        :param pulumi.Input[str] support_url: URL of the support page for the application.
        :param pulumi.Input[str] terms_of_service_url: URL of the terms of service statement for the application.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _ApplicationRegistrationState.__new__(_ApplicationRegistrationState)

        __props__.__dict__["client_id"] = client_id
        __props__.__dict__["description"] = description
        __props__.__dict__["disabled_by_microsoft"] = disabled_by_microsoft
        __props__.__dict__["display_name"] = display_name
        __props__.__dict__["group_membership_claims"] = group_membership_claims
        __props__.__dict__["homepage_url"] = homepage_url
        __props__.__dict__["implicit_access_token_issuance_enabled"] = implicit_access_token_issuance_enabled
        __props__.__dict__["implicit_id_token_issuance_enabled"] = implicit_id_token_issuance_enabled
        __props__.__dict__["logout_url"] = logout_url
        __props__.__dict__["marketing_url"] = marketing_url
        __props__.__dict__["notes"] = notes
        __props__.__dict__["object_id"] = object_id
        __props__.__dict__["privacy_statement_url"] = privacy_statement_url
        __props__.__dict__["publisher_domain"] = publisher_domain
        __props__.__dict__["requested_access_token_version"] = requested_access_token_version
        __props__.__dict__["service_management_reference"] = service_management_reference
        __props__.__dict__["sign_in_audience"] = sign_in_audience
        __props__.__dict__["support_url"] = support_url
        __props__.__dict__["terms_of_service_url"] = terms_of_service_url
        return ApplicationRegistration(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="clientId")
    def client_id(self) -> pulumi.Output[str]:
        """
        The Client ID for the application, which is globally unique.
        """
        return pulumi.get(self, "client_id")

    @property
    @pulumi.getter
    def description(self) -> pulumi.Output[Optional[str]]:
        """
        A description of the application, as shown to end users.
        """
        return pulumi.get(self, "description")

    @property
    @pulumi.getter(name="disabledByMicrosoft")
    def disabled_by_microsoft(self) -> pulumi.Output[str]:
        """
        Whether Microsoft has disabled the registered application. If the application is disabled, this will be a string indicating the status/reason, e.g. `DisabledDueToViolationOfServicesAgreement`
        """
        return pulumi.get(self, "disabled_by_microsoft")

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> pulumi.Output[str]:
        """
        The display name for the application.
        """
        return pulumi.get(self, "display_name")

    @property
    @pulumi.getter(name="groupMembershipClaims")
    def group_membership_claims(self) -> pulumi.Output[Optional[Sequence[str]]]:
        """
        Configures the `groups` claim issued in a user or OAuth access token that the app expects. Possible values are `None`, `SecurityGroup`, `DirectoryRole`, `ApplicationGroup` or `All`.
        """
        return pulumi.get(self, "group_membership_claims")

    @property
    @pulumi.getter(name="homepageUrl")
    def homepage_url(self) -> pulumi.Output[Optional[str]]:
        """
        Home page or landing page of the application.
        """
        return pulumi.get(self, "homepage_url")

    @property
    @pulumi.getter(name="implicitAccessTokenIssuanceEnabled")
    def implicit_access_token_issuance_enabled(self) -> pulumi.Output[Optional[bool]]:
        """
        Whether this web application can request an access token using OAuth implicit flow.
        """
        return pulumi.get(self, "implicit_access_token_issuance_enabled")

    @property
    @pulumi.getter(name="implicitIdTokenIssuanceEnabled")
    def implicit_id_token_issuance_enabled(self) -> pulumi.Output[Optional[bool]]:
        """
        Whether this web application can request an ID token using OAuth implicit flow.
        """
        return pulumi.get(self, "implicit_id_token_issuance_enabled")

    @property
    @pulumi.getter(name="logoutUrl")
    def logout_url(self) -> pulumi.Output[Optional[str]]:
        """
        The URL that will be used by Microsoft's authorization service to sign out a user using front-channel, back-channel or SAML logout protocols.
        """
        return pulumi.get(self, "logout_url")

    @property
    @pulumi.getter(name="marketingUrl")
    def marketing_url(self) -> pulumi.Output[Optional[str]]:
        """
        URL of the marketing page for the application.
        """
        return pulumi.get(self, "marketing_url")

    @property
    @pulumi.getter
    def notes(self) -> pulumi.Output[Optional[str]]:
        """
        User-specified notes relevant for the management of the application.
        """
        return pulumi.get(self, "notes")

    @property
    @pulumi.getter(name="objectId")
    def object_id(self) -> pulumi.Output[str]:
        """
        The object ID of the application within the tenant.
        """
        return pulumi.get(self, "object_id")

    @property
    @pulumi.getter(name="privacyStatementUrl")
    def privacy_statement_url(self) -> pulumi.Output[Optional[str]]:
        """
        URL of the privacy statement for the application.
        """
        return pulumi.get(self, "privacy_statement_url")

    @property
    @pulumi.getter(name="publisherDomain")
    def publisher_domain(self) -> pulumi.Output[str]:
        """
        The verified publisher domain for the application.
        """
        return pulumi.get(self, "publisher_domain")

    @property
    @pulumi.getter(name="requestedAccessTokenVersion")
    def requested_access_token_version(self) -> pulumi.Output[Optional[int]]:
        """
        The access token version expected by this resource. Must be one of `1` or `2`, and must be `2` when `sign_in_audience` is either `AzureADandPersonalMicrosoftAccount` or `PersonalMicrosoftAccount` Defaults to `2`.
        """
        return pulumi.get(self, "requested_access_token_version")

    @property
    @pulumi.getter(name="serviceManagementReference")
    def service_management_reference(self) -> pulumi.Output[Optional[str]]:
        """
        References application context information from a Service or Asset Management database.
        """
        return pulumi.get(self, "service_management_reference")

    @property
    @pulumi.getter(name="signInAudience")
    def sign_in_audience(self) -> pulumi.Output[Optional[str]]:
        """
        The Microsoft account types that are supported for the current application. Must be one of `AzureADMyOrg`, `AzureADMultipleOrgs`, `AzureADandPersonalMicrosoftAccount` or `PersonalMicrosoftAccount`. Defaults to `AzureADMyOrg`.
        """
        return pulumi.get(self, "sign_in_audience")

    @property
    @pulumi.getter(name="supportUrl")
    def support_url(self) -> pulumi.Output[Optional[str]]:
        """
        URL of the support page for the application.
        """
        return pulumi.get(self, "support_url")

    @property
    @pulumi.getter(name="termsOfServiceUrl")
    def terms_of_service_url(self) -> pulumi.Output[Optional[str]]:
        """
        URL of the terms of service statement for the application.
        """
        return pulumi.get(self, "terms_of_service_url")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities

__all__ = [
    'GetDirectoryObjectResult',
    'AwaitableGetDirectoryObjectResult',
    'get_directory_object',
    'get_directory_object_output',
]

@pulumi.output_type
class GetDirectoryObjectResult:
    """
    A collection of values returned by getDirectoryObject.
    """
    def __init__(__self__, id=None, object_id=None, type=None):
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if object_id and not isinstance(object_id, str):
            raise TypeError("Expected argument 'object_id' to be a str")
        pulumi.set(__self__, "object_id", object_id)
        if type and not isinstance(type, str):
            raise TypeError("Expected argument 'type' to be a str")
        pulumi.set(__self__, "type", type)

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter(name="objectId")
    def object_id(self) -> str:
        return pulumi.get(self, "object_id")

    @property
    @pulumi.getter
    def type(self) -> str:
        return pulumi.get(self, "type")


class AwaitableGetDirectoryObjectResult(GetDirectoryObjectResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetDirectoryObjectResult(
            id=self.id,
            object_id=self.object_id,
            type=self.type)


def get_directory_object(object_id: Optional[str] = None,
                         opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetDirectoryObjectResult:
    """
    Retrieves the OData type for a generic directory object having the provided object ID.

    ## API Permissions

    The following API permissions are required in order to use this data source.

    When authenticated with a service principal, this data source requires either `User.Read.All`, `Group.Read.All` or `Directory.Read.All`, depending on the type of object being queried.

    When authenticated with a user principal, this data source does not require any additional roles.

    ## Example Usage

    *Look up and output type of object by ID*
    ```python
    import pulumi
    import pulumi_azuread as azuread

    example = azuread.get_directory_object(object_id="00000000-0000-0000-0000-000000000000")
    pulumi.export("objectType", example.type)
    ```

    ## Attributes Reference

    The following attributes are exported:

    * `object_id` - The object ID of the directory object.
    * `type` - The shortened OData type of the directory object. Possible values include: `Group`, `User` or `ServicePrincipal`.


    :param str object_id: Specifies the Object ID of the directory object to look up.
    """
    __args__ = dict()
    __args__['objectId'] = object_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('azuread:index/getDirectoryObject:getDirectoryObject', __args__, opts=opts, typ=GetDirectoryObjectResult).value

    return AwaitableGetDirectoryObjectResult(
        id=pulumi.get(__ret__, 'id'),
        object_id=pulumi.get(__ret__, 'object_id'),
        type=pulumi.get(__ret__, 'type'))
def get_directory_object_output(object_id: Optional[pulumi.Input[str]] = None,
                                opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetDirectoryObjectResult]:
    """
    Retrieves the OData type for a generic directory object having the provided object ID.

    ## API Permissions

    The following API permissions are required in order to use this data source.

    When authenticated with a service principal, this data source requires either `User.Read.All`, `Group.Read.All` or `Directory.Read.All`, depending on the type of object being queried.

    When authenticated with a user principal, this data source does not require any additional roles.

    ## Example Usage

    *Look up and output type of object by ID*
    ```python
    import pulumi
    import pulumi_azuread as azuread

    example = azuread.get_directory_object(object_id="00000000-0000-0000-0000-000000000000")
    pulumi.export("objectType", example.type)
    ```

    ## Attributes Reference

    The following attributes are exported:

    * `object_id` - The object ID of the directory object.
    * `type` - The shortened OData type of the directory object. Possible values include: `Group`, `User` or `ServicePrincipal`.


    :param str object_id: Specifies the Object ID of the directory object to look up.
    """
    __args__ = dict()
    __args__['objectId'] = object_id
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('azuread:index/getDirectoryObject:getDirectoryObject', __args__, opts=opts, typ=GetDirectoryObjectResult)
    return __ret__.apply(lambda __response__: GetDirectoryObjectResult(
        id=pulumi.get(__response__, 'id'),
        object_id=pulumi.get(__response__, 'object_id'),
        type=pulumi.get(__response__, 'type')))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities

__all__ = ['ApplicationKnownClientsArgs', 'ApplicationKnownClients']

@pulumi.input_type
class ApplicationKnownClientsArgs:
    def __init__(__self__, *,
                 application_id: pulumi.Input[str],
                 known_client_ids: pulumi.Input[Sequence[pulumi.Input[str]]]):
        """
        The set of arguments for constructing a ApplicationKnownClients resource.
        :param pulumi.Input[str] application_id: The resource ID of the application registration. Changing this forces a new resource to be created.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] known_client_ids: A set of client IDs for the known applications.
        """
        pulumi.set(__self__, "application_id", application_id)
        pulumi.set(__self__, "known_client_ids", known_client_ids)

    @property
    @pulumi.getter(name="applicationId")
    def application_id(self) -> pulumi.Input[str]:
        """
        The resource ID of the application registration. Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "application_id")

    @application_id.setter
    def application_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "application_id", value)

    @property
    @pulumi.getter(name="knownClientIds")
    def known_client_ids(self) -> pulumi.Input[Sequence[pulumi.Input[str]]]:
        """
        A set of client IDs for the known applications.
        """
        return pulumi.get(self, "known_client_ids")

    @known_client_ids.setter
    def known_client_ids(self, value: pulumi.Input[Sequence[pulumi.Input[str]]]):
        pulumi.set(self, "known_client_ids", value)


@pulumi.input_type
class _ApplicationKnownClientsState:
    def __init__(__self__, *,
                 application_id: Optional[pulumi.Input[str]] = None,
                 known_client_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None):
        """
        Input properties used for looking up and filtering ApplicationKnownClients resources.
        :param pulumi.Input[str] application_id: The resource ID of the application registration. Changing this forces a new resource to be created.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] known_client_ids: A set of client IDs for the known applications.
        """
        if application_id is not None:
            pulumi.set(__self__, "application_id", application_id)
        if known_client_ids is not None:
            pulumi.set(__self__, "known_client_ids", known_client_ids)

    @property
    @pulumi.getter(name="applicationId")
    def application_id(self) -> Optional[pulumi.Input[str]]:
        """
        The resource ID of the application registration. Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "application_id")

    @application_id.setter
    def application_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "application_id", value)

    @property
    @pulumi.getter(name="knownClientIds")
    def known_client_ids(self) -> Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]:
        """
        A set of client IDs for the known applications.
        """
        return pulumi.get(self, "known_client_ids")

    @known_client_ids.setter
    def known_client_ids(self, value: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]]):
        pulumi.set(self, "known_client_ids", value)


class ApplicationKnownClients(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 application_id: Optional[pulumi.Input[str]] = None,
                 known_client_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 __props__=None):
        """
        ## Example Usage

        ```python
        import pulumi
        import pulumi_azuread as azuread

        example = azuread.ApplicationRegistration("example", display_name="example")
        client = azuread.ApplicationRegistration("client", display_name="example client")
        example_application_known_clients = azuread.ApplicationKnownClients("example",
            application_id=example.id,
            known_client_ids=[client.client_id])
        ```

        ## Import

        Application Known Clients can be imported using the object ID of the application in the following format.

        ```sh
        $ pulumi import azuread:index/applicationKnownClients:ApplicationKnownClients example /applications/00000000-0000-0000-0000-000000000000/knownClients
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] application_id: The resource ID of the application registration. Changing this forces a new resource to be created.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] known_client_ids: A set of client IDs for the known applications.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: ApplicationKnownClientsArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        ## Example Usage

        ```python
        import pulumi
        import pulumi_azuread as azuread

        example = azuread.ApplicationRegistration("example", display_name="example")
        client = azuread.ApplicationRegistration("client", display_name="example client")
        example_application_known_clients = azuread.ApplicationKnownClients("example",
            application_id=example.id,
            known_client_ids=[client.client_id])
        ```

        ## Import

        Application Known Clients can be imported using the object ID of the application in the following format.

        ```sh
        $ pulumi import azuread:index/applicationKnownClients:ApplicationKnownClients example /applications/00000000-0000-0000-0000-000000000000/knownClients
        ```

        :param str resource_name: The name of the resource.
        :param ApplicationKnownClientsArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ApplicationKnownClientsArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 application_id: Optional[pulumi.Input[str]] = None,
                 known_client_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ApplicationKnownClientsArgs.__new__(ApplicationKnownClientsArgs)

            if application_id is None and not opts.urn:
                raise TypeError("Missing required property 'application_id'")
            __props__.__dict__["application_id"] = application_id
            if known_client_ids is None and not opts.urn:
                raise TypeError("Missing required property 'known_client_ids'")
            __props__.__dict__["known_client_ids"] = known_client_ids
        super(ApplicationKnownClients, __self__).__init__(
            'azuread:index/applicationKnownClients:ApplicationKnownClients',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            application_id: Optional[pulumi.Input[str]] = None,
            known_client_ids: Optional[pulumi.Input[Sequence[pulumi.Input[str]]]] = None) -> 'ApplicationKnownClients':
        """
        Get an existing ApplicationKnownClients resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] application_id: The resource ID of the application registration. Changing this forces a new resource to be created.
        :param pulumi.Input[Sequence[pulumi.Input[str]]] known_client_ids: A set of client IDs for the known applications.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _ApplicationKnownClientsState.__new__(_ApplicationKnownClientsState)

        __props__.__dict__["application_id"] = application_id
        __props__.__dict__["known_client_ids"] = known_client_ids
        return ApplicationKnownClients(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="applicationId")
    def application_id(self) -> pulumi.Output[str]:
        """
        The resource ID of the application registration. Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "application_id")

    @property
    @pulumi.getter(name="knownClientIds")
    def known_client_ids(self) -> pulumi.Output[Sequence[str]]:
        """
        A set of client IDs for the known applications.
        """
        return pulumi.get(self, "known_client_ids")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities

__all__ = [
    'GetAdministrativeUnitResult',
    'AwaitableGetAdministrativeUnitResult',
    'get_administrative_unit',
    'get_administrative_unit_output',
]

@pulumi.output_type
class GetAdministrativeUnitResult:
    """
    A collection of values returned by getAdministrativeUnit.
    """
    def __init__(__self__, description=None, display_name=None, id=None, members=None, object_id=None, visibility=None):
        if description and not isinstance(description, str):
            raise TypeError("Expected argument 'description' to be a str")
        pulumi.set(__self__, "description", description)
        if display_name and not isinstance(display_name, str):
            raise TypeError("Expected argument 'display_name' to be a str")
        pulumi.set(__self__, "display_name", display_name)
        if id and not isinstance(id, str):
            raise TypeError("Expected argument 'id' to be a str")
        pulumi.set(__self__, "id", id)
        if members and not isinstance(members, list):
            raise TypeError("Expected argument 'members' to be a list")
        pulumi.set(__self__, "members", members)
        if object_id and not isinstance(object_id, str):
            raise TypeError("Expected argument 'object_id' to be a str")
        pulumi.set(__self__, "object_id", object_id)
        if visibility and not isinstance(visibility, str):
            raise TypeError("Expected argument 'visibility' to be a str")
        pulumi.set(__self__, "visibility", visibility)

    @property
    @pulumi.getter
    def description(self) -> str:
        """
        The description of the administrative unit.
        """
        return pulumi.get(self, "description")

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> str:
        """
        The display name of the administrative unit.
        """
        return pulumi.get(self, "display_name")

    @property
    @pulumi.getter
    def id(self) -> str:
        """
        The provider-assigned unique ID for this managed resource.
        """
        return pulumi.get(self, "id")

    @property
    @pulumi.getter
    def members(self) -> Sequence[str]:
        """
        A list of object IDs of members who are present in this administrative unit.
        """
        return pulumi.get(self, "members")

    @property
    @pulumi.getter(name="objectId")
    def object_id(self) -> str:
        """
        The object ID of the administrative unit.
        """
        return pulumi.get(self, "object_id")

    @property
    @pulumi.getter
    def visibility(self) -> str:
        """
        Whether the administrative unit _and_ its members are hidden or publicly viewable in the directory. One of: `Hiddenmembership` or `Public`.
        """
        return pulumi.get(self, "visibility")


class AwaitableGetAdministrativeUnitResult(GetAdministrativeUnitResult):
    # pylint: disable=using-constant-test
    def __await__(self):
        if False:
            yield self
        return GetAdministrativeUnitResult(
            description=self.description,
            display_name=self.display_name,
            id=self.id,
            members=self.members,
            object_id=self.object_id,
            visibility=self.visibility)


def get_administrative_unit(display_name: Optional[str] = None,
                            object_id: Optional[str] = None,
                            opts: Optional[pulumi.InvokeOptions] = None) -> AwaitableGetAdministrativeUnitResult:
    """
    Gets information about an adminisrative unit in Azure Active Directory.

    ## API Permissions

    The following API permissions are required in order to use this data source.

    When authenticated with a service principal, this data source requires one of the following application roles: `AdministrativeUnit.Read.All` or `Directory.Read.All`

    When authenticated with a user principal, this data source does not require any additional roles.

    ## Example Usage

    ### By Group Display Name)

    *Look up by display name*
    ```python
    import pulumi
    import pulumi_azuread as azuread

    example = azuread.get_administrative_unit(display_name="Example-AU")
    ```

    *Look up by object ID*
    ```python
    import pulumi
    import pulumi_azuread as azuread

    example = azuread.get_administrative_unit(object_id="00000000-0000-0000-0000-000000000000")
    ```


    :param str display_name: Specifies the display name of the administrative unit.
    :param str object_id: Specifies the object ID of the administrative unit.
           
           > One of `display_name` or `object_id` must be specified.
    """
    __args__ = dict()
    __args__['displayName'] = display_name
    __args__['objectId'] = object_id
    opts = pulumi.InvokeOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke('azuread:index/getAdministrativeUnit:getAdministrativeUnit', __args__, opts=opts, typ=GetAdministrativeUnitResult).value

    return AwaitableGetAdministrativeUnitResult(
        description=pulumi.get(__ret__, 'description'),
        display_name=pulumi.get(__ret__, 'display_name'),
        id=pulumi.get(__ret__, 'id'),
        members=pulumi.get(__ret__, 'members'),
        object_id=pulumi.get(__ret__, 'object_id'),
        visibility=pulumi.get(__ret__, 'visibility'))
def get_administrative_unit_output(display_name: Optional[pulumi.Input[Optional[str]]] = None,
                                   object_id: Optional[pulumi.Input[Optional[str]]] = None,
                                   opts: Optional[Union[pulumi.InvokeOptions, pulumi.InvokeOutputOptions]] = None) -> pulumi.Output[GetAdministrativeUnitResult]:
    """
    Gets information about an adminisrative unit in Azure Active Directory.

    ## API Permissions

    The following API permissions are required in order to use this data source.

    When authenticated with a service principal, this data source requires one of the following application roles: `AdministrativeUnit.Read.All` or `Directory.Read.All`

    When authenticated with a user principal, this data source does not require any additional roles.

    ## Example Usage

    ### By Group Display Name)

    *Look up by display name*
    ```python
    import pulumi
    import pulumi_azuread as azuread

    example = azuread.get_administrative_unit(display_name="Example-AU")
    ```

    *Look up by object ID*
    ```python
    import pulumi
    import pulumi_azuread as azuread

    example = azuread.get_administrative_unit(object_id="00000000-0000-0000-0000-000000000000")
    ```


    :param str display_name: Specifies the display name of the administrative unit.
    :param str object_id: Specifies the object ID of the administrative unit.
           
           > One of `display_name` or `object_id` must be specified.
    """
    __args__ = dict()
    __args__['displayName'] = display_name
    __args__['objectId'] = object_id
    opts = pulumi.InvokeOutputOptions.merge(_utilities.get_invoke_opts_defaults(), opts)
    __ret__ = pulumi.runtime.invoke_output('azuread:index/getAdministrativeUnit:getAdministrativeUnit', __args__, opts=opts, typ=GetAdministrativeUnitResult)
    return __ret__.apply(lambda __response__: GetAdministrativeUnitResult(
        description=pulumi.get(__response__, 'description'),
        display_name=pulumi.get(__response__, 'display_name'),
        id=pulumi.get(__response__, 'id'),
        members=pulumi.get(__response__, 'members'),
        object_id=pulumi.get(__response__, 'object_id'),
        visibility=pulumi.get(__response__, 'visibility')))



# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities

__all__ = ['PrivilegedAccessGroupEligibilityScheduleArgs', 'PrivilegedAccessGroupEligibilitySchedule']

@pulumi.input_type
class PrivilegedAccessGroupEligibilityScheduleArgs:
    def __init__(__self__, *,
                 assignment_type: pulumi.Input[str],
                 group_id: pulumi.Input[str],
                 principal_id: pulumi.Input[str],
                 duration: Optional[pulumi.Input[str]] = None,
                 expiration_date: Optional[pulumi.Input[str]] = None,
                 justification: Optional[pulumi.Input[str]] = None,
                 permanent_assignment: Optional[pulumi.Input[bool]] = None,
                 start_date: Optional[pulumi.Input[str]] = None,
                 ticket_number: Optional[pulumi.Input[str]] = None,
                 ticket_system: Optional[pulumi.Input[str]] = None):
        """
        The set of arguments for constructing a PrivilegedAccessGroupEligibilitySchedule resource.
        :param pulumi.Input[str] assignment_type: The type of assignment to the group. Can be either `member` or `owner`.
        :param pulumi.Input[str] group_id: The Object ID of the Azure AD group to which the principal will be assigned.
        :param pulumi.Input[str] principal_id: The Object ID of the principal to be assigned to the above group. Can be either a user or a group.
        :param pulumi.Input[str] duration: The duration that this assignment is valid for, formatted as an ISO8601 duration (e.g. P30D for 30 days, PT3H for three hours).
        :param pulumi.Input[str] expiration_date: The date that this assignment expires, formatted as an RFC3339 date string (e.g. 2018-01-01T01:02:03Z).
        :param pulumi.Input[str] justification: The justification for this assignment. May be required by the role policy.
        :param pulumi.Input[bool] permanent_assignment: Is this assigment permanently valid.
               
               At least one of `expiration_date`, `duration`, or `permanent_assignment` must be supplied. The role policy may limit the maximum duration which can be supplied.
        :param pulumi.Input[str] start_date: The date from which this assignment is valid, formatted as an RFC3339 date string (e.g. 2018-01-01T01:02:03Z). If not provided, the assignment is immediately valid.
        :param pulumi.Input[str] ticket_number: The ticket number in the ticket system approving this assignment. May be required by the role policy.
        :param pulumi.Input[str] ticket_system: The ticket system containing the ticket number approving this assignment. May be required by the role policy.
        """
        pulumi.set(__self__, "assignment_type", assignment_type)
        pulumi.set(__self__, "group_id", group_id)
        pulumi.set(__self__, "principal_id", principal_id)
        if duration is not None:
            pulumi.set(__self__, "duration", duration)
        if expiration_date is not None:
            pulumi.set(__self__, "expiration_date", expiration_date)
        if justification is not None:
            pulumi.set(__self__, "justification", justification)
        if permanent_assignment is not None:
            pulumi.set(__self__, "permanent_assignment", permanent_assignment)
        if start_date is not None:
            pulumi.set(__self__, "start_date", start_date)
        if ticket_number is not None:
            pulumi.set(__self__, "ticket_number", ticket_number)
        if ticket_system is not None:
            pulumi.set(__self__, "ticket_system", ticket_system)

    @property
    @pulumi.getter(name="assignmentType")
    def assignment_type(self) -> pulumi.Input[str]:
        """
        The type of assignment to the group. Can be either `member` or `owner`.
        """
        return pulumi.get(self, "assignment_type")

    @assignment_type.setter
    def assignment_type(self, value: pulumi.Input[str]):
        pulumi.set(self, "assignment_type", value)

    @property
    @pulumi.getter(name="groupId")
    def group_id(self) -> pulumi.Input[str]:
        """
        The Object ID of the Azure AD group to which the principal will be assigned.
        """
        return pulumi.get(self, "group_id")

    @group_id.setter
    def group_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "group_id", value)

    @property
    @pulumi.getter(name="principalId")
    def principal_id(self) -> pulumi.Input[str]:
        """
        The Object ID of the principal to be assigned to the above group. Can be either a user or a group.
        """
        return pulumi.get(self, "principal_id")

    @principal_id.setter
    def principal_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "principal_id", value)

    @property
    @pulumi.getter
    def duration(self) -> Optional[pulumi.Input[str]]:
        """
        The duration that this assignment is valid for, formatted as an ISO8601 duration (e.g. P30D for 30 days, PT3H for three hours).
        """
        return pulumi.get(self, "duration")

    @duration.setter
    def duration(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "duration", value)

    @property
    @pulumi.getter(name="expirationDate")
    def expiration_date(self) -> Optional[pulumi.Input[str]]:
        """
        The date that this assignment expires, formatted as an RFC3339 date string (e.g. 2018-01-01T01:02:03Z).
        """
        return pulumi.get(self, "expiration_date")

    @expiration_date.setter
    def expiration_date(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "expiration_date", value)

    @property
    @pulumi.getter
    def justification(self) -> Optional[pulumi.Input[str]]:
        """
        The justification for this assignment. May be required by the role policy.
        """
        return pulumi.get(self, "justification")

    @justification.setter
    def justification(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "justification", value)

    @property
    @pulumi.getter(name="permanentAssignment")
    def permanent_assignment(self) -> Optional[pulumi.Input[bool]]:
        """
        Is this assigment permanently valid.

        At least one of `expiration_date`, `duration`, or `permanent_assignment` must be supplied. The role policy may limit the maximum duration which can be supplied.
        """
        return pulumi.get(self, "permanent_assignment")

    @permanent_assignment.setter
    def permanent_assignment(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "permanent_assignment", value)

    @property
    @pulumi.getter(name="startDate")
    def start_date(self) -> Optional[pulumi.Input[str]]:
        """
        The date from which this assignment is valid, formatted as an RFC3339 date string (e.g. 2018-01-01T01:02:03Z). If not provided, the assignment is immediately valid.
        """
        return pulumi.get(self, "start_date")

    @start_date.setter
    def start_date(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "start_date", value)

    @property
    @pulumi.getter(name="ticketNumber")
    def ticket_number(self) -> Optional[pulumi.Input[str]]:
        """
        The ticket number in the ticket system approving this assignment. May be required by the role policy.
        """
        return pulumi.get(self, "ticket_number")

    @ticket_number.setter
    def ticket_number(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "ticket_number", value)

    @property
    @pulumi.getter(name="ticketSystem")
    def ticket_system(self) -> Optional[pulumi.Input[str]]:
        """
        The ticket system containing the ticket number approving this assignment. May be required by the role policy.
        """
        return pulumi.get(self, "ticket_system")

    @ticket_system.setter
    def ticket_system(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "ticket_system", value)


@pulumi.input_type
class _PrivilegedAccessGroupEligibilityScheduleState:
    def __init__(__self__, *,
                 assignment_type: Optional[pulumi.Input[str]] = None,
                 duration: Optional[pulumi.Input[str]] = None,
                 expiration_date: Optional[pulumi.Input[str]] = None,
                 group_id: Optional[pulumi.Input[str]] = None,
                 justification: Optional[pulumi.Input[str]] = None,
                 permanent_assignment: Optional[pulumi.Input[bool]] = None,
                 principal_id: Optional[pulumi.Input[str]] = None,
                 start_date: Optional[pulumi.Input[str]] = None,
                 status: Optional[pulumi.Input[str]] = None,
                 ticket_number: Optional[pulumi.Input[str]] = None,
                 ticket_system: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering PrivilegedAccessGroupEligibilitySchedule resources.
        :param pulumi.Input[str] assignment_type: The type of assignment to the group. Can be either `member` or `owner`.
        :param pulumi.Input[str] duration: The duration that this assignment is valid for, formatted as an ISO8601 duration (e.g. P30D for 30 days, PT3H for three hours).
        :param pulumi.Input[str] expiration_date: The date that this assignment expires, formatted as an RFC3339 date string (e.g. 2018-01-01T01:02:03Z).
        :param pulumi.Input[str] group_id: The Object ID of the Azure AD group to which the principal will be assigned.
        :param pulumi.Input[str] justification: The justification for this assignment. May be required by the role policy.
        :param pulumi.Input[bool] permanent_assignment: Is this assigment permanently valid.
               
               At least one of `expiration_date`, `duration`, or `permanent_assignment` must be supplied. The role policy may limit the maximum duration which can be supplied.
        :param pulumi.Input[str] principal_id: The Object ID of the principal to be assigned to the above group. Can be either a user or a group.
        :param pulumi.Input[str] start_date: The date from which this assignment is valid, formatted as an RFC3339 date string (e.g. 2018-01-01T01:02:03Z). If not provided, the assignment is immediately valid.
        :param pulumi.Input[str] status: (String) The provisioning status of this request.
        :param pulumi.Input[str] ticket_number: The ticket number in the ticket system approving this assignment. May be required by the role policy.
        :param pulumi.Input[str] ticket_system: The ticket system containing the ticket number approving this assignment. May be required by the role policy.
        """
        if assignment_type is not None:
            pulumi.set(__self__, "assignment_type", assignment_type)
        if duration is not None:
            pulumi.set(__self__, "duration", duration)
        if expiration_date is not None:
            pulumi.set(__self__, "expiration_date", expiration_date)
        if group_id is not None:
            pulumi.set(__self__, "group_id", group_id)
        if justification is not None:
            pulumi.set(__self__, "justification", justification)
        if permanent_assignment is not None:
            pulumi.set(__self__, "permanent_assignment", permanent_assignment)
        if principal_id is not None:
            pulumi.set(__self__, "principal_id", principal_id)
        if start_date is not None:
            pulumi.set(__self__, "start_date", start_date)
        if status is not None:
            pulumi.set(__self__, "status", status)
        if ticket_number is not None:
            pulumi.set(__self__, "ticket_number", ticket_number)
        if ticket_system is not None:
            pulumi.set(__self__, "ticket_system", ticket_system)

    @property
    @pulumi.getter(name="assignmentType")
    def assignment_type(self) -> Optional[pulumi.Input[str]]:
        """
        The type of assignment to the group. Can be either `member` or `owner`.
        """
        return pulumi.get(self, "assignment_type")

    @assignment_type.setter
    def assignment_type(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "assignment_type", value)

    @property
    @pulumi.getter
    def duration(self) -> Optional[pulumi.Input[str]]:
        """
        The duration that this assignment is valid for, formatted as an ISO8601 duration (e.g. P30D for 30 days, PT3H for three hours).
        """
        return pulumi.get(self, "duration")

    @duration.setter
    def duration(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "duration", value)

    @property
    @pulumi.getter(name="expirationDate")
    def expiration_date(self) -> Optional[pulumi.Input[str]]:
        """
        The date that this assignment expires, formatted as an RFC3339 date string (e.g. 2018-01-01T01:02:03Z).
        """
        return pulumi.get(self, "expiration_date")

    @expiration_date.setter
    def expiration_date(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "expiration_date", value)

    @property
    @pulumi.getter(name="groupId")
    def group_id(self) -> Optional[pulumi.Input[str]]:
        """
        The Object ID of the Azure AD group to which the principal will be assigned.
        """
        return pulumi.get(self, "group_id")

    @group_id.setter
    def group_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "group_id", value)

    @property
    @pulumi.getter
    def justification(self) -> Optional[pulumi.Input[str]]:
        """
        The justification for this assignment. May be required by the role policy.
        """
        return pulumi.get(self, "justification")

    @justification.setter
    def justification(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "justification", value)

    @property
    @pulumi.getter(name="permanentAssignment")
    def permanent_assignment(self) -> Optional[pulumi.Input[bool]]:
        """
        Is this assigment permanently valid.

        At least one of `expiration_date`, `duration`, or `permanent_assignment` must be supplied. The role policy may limit the maximum duration which can be supplied.
        """
        return pulumi.get(self, "permanent_assignment")

    @permanent_assignment.setter
    def permanent_assignment(self, value: Optional[pulumi.Input[bool]]):
        pulumi.set(self, "permanent_assignment", value)

    @property
    @pulumi.getter(name="principalId")
    def principal_id(self) -> Optional[pulumi.Input[str]]:
        """
        The Object ID of the principal to be assigned to the above group. Can be either a user or a group.
        """
        return pulumi.get(self, "principal_id")

    @principal_id.setter
    def principal_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "principal_id", value)

    @property
    @pulumi.getter(name="startDate")
    def start_date(self) -> Optional[pulumi.Input[str]]:
        """
        The date from which this assignment is valid, formatted as an RFC3339 date string (e.g. 2018-01-01T01:02:03Z). If not provided, the assignment is immediately valid.
        """
        return pulumi.get(self, "start_date")

    @start_date.setter
    def start_date(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "start_date", value)

    @property
    @pulumi.getter
    def status(self) -> Optional[pulumi.Input[str]]:
        """
        (String) The provisioning status of this request.
        """
        return pulumi.get(self, "status")

    @status.setter
    def status(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "status", value)

    @property
    @pulumi.getter(name="ticketNumber")
    def ticket_number(self) -> Optional[pulumi.Input[str]]:
        """
        The ticket number in the ticket system approving this assignment. May be required by the role policy.
        """
        return pulumi.get(self, "ticket_number")

    @ticket_number.setter
    def ticket_number(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "ticket_number", value)

    @property
    @pulumi.getter(name="ticketSystem")
    def ticket_system(self) -> Optional[pulumi.Input[str]]:
        """
        The ticket system containing the ticket number approving this assignment. May be required by the role policy.
        """
        return pulumi.get(self, "ticket_system")

    @ticket_system.setter
    def ticket_system(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "ticket_system", value)


class PrivilegedAccessGroupEligibilitySchedule(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 assignment_type: Optional[pulumi.Input[str]] = None,
                 duration: Optional[pulumi.Input[str]] = None,
                 expiration_date: Optional[pulumi.Input[str]] = None,
                 group_id: Optional[pulumi.Input[str]] = None,
                 justification: Optional[pulumi.Input[str]] = None,
                 permanent_assignment: Optional[pulumi.Input[bool]] = None,
                 principal_id: Optional[pulumi.Input[str]] = None,
                 start_date: Optional[pulumi.Input[str]] = None,
                 ticket_number: Optional[pulumi.Input[str]] = None,
                 ticket_system: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Manages an eligible assignment to a privileged access group.

        ## API Permissions

        The following API permissions are required in order to use this resource.

        When authenticated with a service principal, this resource requires the `PrivilegedEligibilitySchedule.ReadWrite.AzureADGroup` Microsoft Graph API permissions.

        When authenticated with a user principal, this resource requires `Global Administrator` directory role, or the `Privileged Role Administrator` role in Identity Governance.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_azuread as azuread

        example = azuread.Group("example",
            display_name="group-name",
            security_enabled=True)
        member = azuread.User("member",
            user_principal_name="jdoe@example.com",
            display_name="J. Doe",
            mail_nickname="jdoe",
            password="SecretP@sswd99!")
        example_privileged_access_group_eligibility_schedule = azuread.PrivilegedAccessGroupEligibilitySchedule("example",
            group_id=pim["id"],
            principal_id=member.id,
            assignment_type="member",
            duration="P30D",
            justification="as requested")
        ```

        ## Import

        An assignment schedule can be imported using the schedule ID, e.g.

        ```sh
        $ pulumi import azuread:index/privilegedAccessGroupEligibilitySchedule:PrivilegedAccessGroupEligibilitySchedule example 00000000-0000-0000-0000-000000000000_member_00000000-0000-0000-0000-000000000000
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] assignment_type: The type of assignment to the group. Can be either `member` or `owner`.
        :param pulumi.Input[str] duration: The duration that this assignment is valid for, formatted as an ISO8601 duration (e.g. P30D for 30 days, PT3H for three hours).
        :param pulumi.Input[str] expiration_date: The date that this assignment expires, formatted as an RFC3339 date string (e.g. 2018-01-01T01:02:03Z).
        :param pulumi.Input[str] group_id: The Object ID of the Azure AD group to which the principal will be assigned.
        :param pulumi.Input[str] justification: The justification for this assignment. May be required by the role policy.
        :param pulumi.Input[bool] permanent_assignment: Is this assigment permanently valid.
               
               At least one of `expiration_date`, `duration`, or `permanent_assignment` must be supplied. The role policy may limit the maximum duration which can be supplied.
        :param pulumi.Input[str] principal_id: The Object ID of the principal to be assigned to the above group. Can be either a user or a group.
        :param pulumi.Input[str] start_date: The date from which this assignment is valid, formatted as an RFC3339 date string (e.g. 2018-01-01T01:02:03Z). If not provided, the assignment is immediately valid.
        :param pulumi.Input[str] ticket_number: The ticket number in the ticket system approving this assignment. May be required by the role policy.
        :param pulumi.Input[str] ticket_system: The ticket system containing the ticket number approving this assignment. May be required by the role policy.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: PrivilegedAccessGroupEligibilityScheduleArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Manages an eligible assignment to a privileged access group.

        ## API Permissions

        The following API permissions are required in order to use this resource.

        When authenticated with a service principal, this resource requires the `PrivilegedEligibilitySchedule.ReadWrite.AzureADGroup` Microsoft Graph API permissions.

        When authenticated with a user principal, this resource requires `Global Administrator` directory role, or the `Privileged Role Administrator` role in Identity Governance.

        ## Example Usage

        ```python
        import pulumi
        import pulumi_azuread as azuread

        example = azuread.Group("example",
            display_name="group-name",
            security_enabled=True)
        member = azuread.User("member",
            user_principal_name="jdoe@example.com",
            display_name="J. Doe",
            mail_nickname="jdoe",
            password="SecretP@sswd99!")
        example_privileged_access_group_eligibility_schedule = azuread.PrivilegedAccessGroupEligibilitySchedule("example",
            group_id=pim["id"],
            principal_id=member.id,
            assignment_type="member",
            duration="P30D",
            justification="as requested")
        ```

        ## Import

        An assignment schedule can be imported using the schedule ID, e.g.

        ```sh
        $ pulumi import azuread:index/privilegedAccessGroupEligibilitySchedule:PrivilegedAccessGroupEligibilitySchedule example 00000000-0000-0000-0000-000000000000_member_00000000-0000-0000-0000-000000000000
        ```

        :param str resource_name: The name of the resource.
        :param PrivilegedAccessGroupEligibilityScheduleArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(PrivilegedAccessGroupEligibilityScheduleArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 assignment_type: Optional[pulumi.Input[str]] = None,
                 duration: Optional[pulumi.Input[str]] = None,
                 expiration_date: Optional[pulumi.Input[str]] = None,
                 group_id: Optional[pulumi.Input[str]] = None,
                 justification: Optional[pulumi.Input[str]] = None,
                 permanent_assignment: Optional[pulumi.Input[bool]] = None,
                 principal_id: Optional[pulumi.Input[str]] = None,
                 start_date: Optional[pulumi.Input[str]] = None,
                 ticket_number: Optional[pulumi.Input[str]] = None,
                 ticket_system: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = PrivilegedAccessGroupEligibilityScheduleArgs.__new__(PrivilegedAccessGroupEligibilityScheduleArgs)

            if assignment_type is None and not opts.urn:
                raise TypeError("Missing required property 'assignment_type'")
            __props__.__dict__["assignment_type"] = assignment_type
            __props__.__dict__["duration"] = duration
            __props__.__dict__["expiration_date"] = expiration_date
            if group_id is None and not opts.urn:
                raise TypeError("Missing required property 'group_id'")
            __props__.__dict__["group_id"] = group_id
            __props__.__dict__["justification"] = justification
            __props__.__dict__["permanent_assignment"] = permanent_assignment
            if principal_id is None and not opts.urn:
                raise TypeError("Missing required property 'principal_id'")
            __props__.__dict__["principal_id"] = principal_id
            __props__.__dict__["start_date"] = start_date
            __props__.__dict__["ticket_number"] = ticket_number
            __props__.__dict__["ticket_system"] = ticket_system
            __props__.__dict__["status"] = None
        super(PrivilegedAccessGroupEligibilitySchedule, __self__).__init__(
            'azuread:index/privilegedAccessGroupEligibilitySchedule:PrivilegedAccessGroupEligibilitySchedule',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            assignment_type: Optional[pulumi.Input[str]] = None,
            duration: Optional[pulumi.Input[str]] = None,
            expiration_date: Optional[pulumi.Input[str]] = None,
            group_id: Optional[pulumi.Input[str]] = None,
            justification: Optional[pulumi.Input[str]] = None,
            permanent_assignment: Optional[pulumi.Input[bool]] = None,
            principal_id: Optional[pulumi.Input[str]] = None,
            start_date: Optional[pulumi.Input[str]] = None,
            status: Optional[pulumi.Input[str]] = None,
            ticket_number: Optional[pulumi.Input[str]] = None,
            ticket_system: Optional[pulumi.Input[str]] = None) -> 'PrivilegedAccessGroupEligibilitySchedule':
        """
        Get an existing PrivilegedAccessGroupEligibilitySchedule resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] assignment_type: The type of assignment to the group. Can be either `member` or `owner`.
        :param pulumi.Input[str] duration: The duration that this assignment is valid for, formatted as an ISO8601 duration (e.g. P30D for 30 days, PT3H for three hours).
        :param pulumi.Input[str] expiration_date: The date that this assignment expires, formatted as an RFC3339 date string (e.g. 2018-01-01T01:02:03Z).
        :param pulumi.Input[str] group_id: The Object ID of the Azure AD group to which the principal will be assigned.
        :param pulumi.Input[str] justification: The justification for this assignment. May be required by the role policy.
        :param pulumi.Input[bool] permanent_assignment: Is this assigment permanently valid.
               
               At least one of `expiration_date`, `duration`, or `permanent_assignment` must be supplied. The role policy may limit the maximum duration which can be supplied.
        :param pulumi.Input[str] principal_id: The Object ID of the principal to be assigned to the above group. Can be either a user or a group.
        :param pulumi.Input[str] start_date: The date from which this assignment is valid, formatted as an RFC3339 date string (e.g. 2018-01-01T01:02:03Z). If not provided, the assignment is immediately valid.
        :param pulumi.Input[str] status: (String) The provisioning status of this request.
        :param pulumi.Input[str] ticket_number: The ticket number in the ticket system approving this assignment. May be required by the role policy.
        :param pulumi.Input[str] ticket_system: The ticket system containing the ticket number approving this assignment. May be required by the role policy.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _PrivilegedAccessGroupEligibilityScheduleState.__new__(_PrivilegedAccessGroupEligibilityScheduleState)

        __props__.__dict__["assignment_type"] = assignment_type
        __props__.__dict__["duration"] = duration
        __props__.__dict__["expiration_date"] = expiration_date
        __props__.__dict__["group_id"] = group_id
        __props__.__dict__["justification"] = justification
        __props__.__dict__["permanent_assignment"] = permanent_assignment
        __props__.__dict__["principal_id"] = principal_id
        __props__.__dict__["start_date"] = start_date
        __props__.__dict__["status"] = status
        __props__.__dict__["ticket_number"] = ticket_number
        __props__.__dict__["ticket_system"] = ticket_system
        return PrivilegedAccessGroupEligibilitySchedule(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="assignmentType")
    def assignment_type(self) -> pulumi.Output[str]:
        """
        The type of assignment to the group. Can be either `member` or `owner`.
        """
        return pulumi.get(self, "assignment_type")

    @property
    @pulumi.getter
    def duration(self) -> pulumi.Output[Optional[str]]:
        """
        The duration that this assignment is valid for, formatted as an ISO8601 duration (e.g. P30D for 30 days, PT3H for three hours).
        """
        return pulumi.get(self, "duration")

    @property
    @pulumi.getter(name="expirationDate")
    def expiration_date(self) -> pulumi.Output[str]:
        """
        The date that this assignment expires, formatted as an RFC3339 date string (e.g. 2018-01-01T01:02:03Z).
        """
        return pulumi.get(self, "expiration_date")

    @property
    @pulumi.getter(name="groupId")
    def group_id(self) -> pulumi.Output[str]:
        """
        The Object ID of the Azure AD group to which the principal will be assigned.
        """
        return pulumi.get(self, "group_id")

    @property
    @pulumi.getter
    def justification(self) -> pulumi.Output[Optional[str]]:
        """
        The justification for this assignment. May be required by the role policy.
        """
        return pulumi.get(self, "justification")

    @property
    @pulumi.getter(name="permanentAssignment")
    def permanent_assignment(self) -> pulumi.Output[bool]:
        """
        Is this assigment permanently valid.

        At least one of `expiration_date`, `duration`, or `permanent_assignment` must be supplied. The role policy may limit the maximum duration which can be supplied.
        """
        return pulumi.get(self, "permanent_assignment")

    @property
    @pulumi.getter(name="principalId")
    def principal_id(self) -> pulumi.Output[str]:
        """
        The Object ID of the principal to be assigned to the above group. Can be either a user or a group.
        """
        return pulumi.get(self, "principal_id")

    @property
    @pulumi.getter(name="startDate")
    def start_date(self) -> pulumi.Output[str]:
        """
        The date from which this assignment is valid, formatted as an RFC3339 date string (e.g. 2018-01-01T01:02:03Z). If not provided, the assignment is immediately valid.
        """
        return pulumi.get(self, "start_date")

    @property
    @pulumi.getter
    def status(self) -> pulumi.Output[str]:
        """
        (String) The provisioning status of this request.
        """
        return pulumi.get(self, "status")

    @property
    @pulumi.getter(name="ticketNumber")
    def ticket_number(self) -> pulumi.Output[Optional[str]]:
        """
        The ticket number in the ticket system approving this assignment. May be required by the role policy.
        """
        return pulumi.get(self, "ticket_number")

    @property
    @pulumi.getter(name="ticketSystem")
    def ticket_system(self) -> pulumi.Output[Optional[str]]:
        """
        The ticket system containing the ticket number approving this assignment. May be required by the role policy.
        """
        return pulumi.get(self, "ticket_system")




# coding=utf-8
# *** WARNING: this file was generated by the Pulumi Terraform Bridge (tfgen) Tool. ***
# *** Do not edit by hand unless you're certain you know what you are doing! ***

import copy
import warnings
import sys
import pulumi
import pulumi.runtime
from typing import Any, Mapping, Optional, Sequence, Union, overload
if sys.version_info >= (3, 11):
    from typing import NotRequired, TypedDict, TypeAlias
else:
    from typing_extensions import NotRequired, TypedDict, TypeAlias
from . import _utilities

__all__ = ['ApplicationFromTemplateArgs', 'ApplicationFromTemplate']

@pulumi.input_type
class ApplicationFromTemplateArgs:
    def __init__(__self__, *,
                 display_name: pulumi.Input[str],
                 template_id: pulumi.Input[str]):
        """
        The set of arguments for constructing a ApplicationFromTemplate resource.
        :param pulumi.Input[str] display_name: The display name for the application.
        :param pulumi.Input[str] template_id: Unique ID for a templated application in the Azure AD App Gallery, from which to create the application. Changing this forces a new resource to be created.
        """
        pulumi.set(__self__, "display_name", display_name)
        pulumi.set(__self__, "template_id", template_id)

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> pulumi.Input[str]:
        """
        The display name for the application.
        """
        return pulumi.get(self, "display_name")

    @display_name.setter
    def display_name(self, value: pulumi.Input[str]):
        pulumi.set(self, "display_name", value)

    @property
    @pulumi.getter(name="templateId")
    def template_id(self) -> pulumi.Input[str]:
        """
        Unique ID for a templated application in the Azure AD App Gallery, from which to create the application. Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "template_id")

    @template_id.setter
    def template_id(self, value: pulumi.Input[str]):
        pulumi.set(self, "template_id", value)


@pulumi.input_type
class _ApplicationFromTemplateState:
    def __init__(__self__, *,
                 application_id: Optional[pulumi.Input[str]] = None,
                 application_object_id: Optional[pulumi.Input[str]] = None,
                 display_name: Optional[pulumi.Input[str]] = None,
                 service_principal_id: Optional[pulumi.Input[str]] = None,
                 service_principal_object_id: Optional[pulumi.Input[str]] = None,
                 template_id: Optional[pulumi.Input[str]] = None):
        """
        Input properties used for looking up and filtering ApplicationFromTemplate resources.
        :param pulumi.Input[str] application_id: The resource ID for the application.
        :param pulumi.Input[str] application_object_id: The object ID for the application.
        :param pulumi.Input[str] display_name: The display name for the application.
        :param pulumi.Input[str] service_principal_id: The resource ID for the service principal.
        :param pulumi.Input[str] service_principal_object_id: The object ID for the service principal.
        :param pulumi.Input[str] template_id: Unique ID for a templated application in the Azure AD App Gallery, from which to create the application. Changing this forces a new resource to be created.
        """
        if application_id is not None:
            pulumi.set(__self__, "application_id", application_id)
        if application_object_id is not None:
            pulumi.set(__self__, "application_object_id", application_object_id)
        if display_name is not None:
            pulumi.set(__self__, "display_name", display_name)
        if service_principal_id is not None:
            pulumi.set(__self__, "service_principal_id", service_principal_id)
        if service_principal_object_id is not None:
            pulumi.set(__self__, "service_principal_object_id", service_principal_object_id)
        if template_id is not None:
            pulumi.set(__self__, "template_id", template_id)

    @property
    @pulumi.getter(name="applicationId")
    def application_id(self) -> Optional[pulumi.Input[str]]:
        """
        The resource ID for the application.
        """
        return pulumi.get(self, "application_id")

    @application_id.setter
    def application_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "application_id", value)

    @property
    @pulumi.getter(name="applicationObjectId")
    def application_object_id(self) -> Optional[pulumi.Input[str]]:
        """
        The object ID for the application.
        """
        return pulumi.get(self, "application_object_id")

    @application_object_id.setter
    def application_object_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "application_object_id", value)

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> Optional[pulumi.Input[str]]:
        """
        The display name for the application.
        """
        return pulumi.get(self, "display_name")

    @display_name.setter
    def display_name(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "display_name", value)

    @property
    @pulumi.getter(name="servicePrincipalId")
    def service_principal_id(self) -> Optional[pulumi.Input[str]]:
        """
        The resource ID for the service principal.
        """
        return pulumi.get(self, "service_principal_id")

    @service_principal_id.setter
    def service_principal_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "service_principal_id", value)

    @property
    @pulumi.getter(name="servicePrincipalObjectId")
    def service_principal_object_id(self) -> Optional[pulumi.Input[str]]:
        """
        The object ID for the service principal.
        """
        return pulumi.get(self, "service_principal_object_id")

    @service_principal_object_id.setter
    def service_principal_object_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "service_principal_object_id", value)

    @property
    @pulumi.getter(name="templateId")
    def template_id(self) -> Optional[pulumi.Input[str]]:
        """
        Unique ID for a templated application in the Azure AD App Gallery, from which to create the application. Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "template_id")

    @template_id.setter
    def template_id(self, value: Optional[pulumi.Input[str]]):
        pulumi.set(self, "template_id", value)


class ApplicationFromTemplate(pulumi.CustomResource):
    @overload
    def __init__(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 display_name: Optional[pulumi.Input[str]] = None,
                 template_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        """
        Creates an application registration and associated service principal from a gallery template.

        > The Application resource can also be used to instantiate a gallery application, however unlike the `Application` resource, this resource does not attempt to manage any properties of the resulting application.

        ## API Permissions

        The following API permissions are required in order to use this resource.

        When authenticated with a service principal, this resource requires one of the following application roles: `Application.ReadWrite.OwnedBy` or `Application.ReadWrite.All`

        When authenticated with a user principal, this resource may require one of the following directory roles: `Application Administrator` or `Global Administrator`

        ## Example Usage

        ```python
        import pulumi
        import pulumi_azuread as azuread

        example = azuread.get_application_template(display_name="Marketo")
        example_application_from_template = azuread.ApplicationFromTemplate("example",
            display_name="Example Application",
            template_id=example.template_id)
        example_get_application = azuread.get_application_output(object_id=example_application_from_template.application_object_id)
        example_get_service_principal = azuread.get_service_principal_output(object_id=example_application_from_template.service_principal_object_id)
        ```

        ## Import

        Templated Applications can be imported using the template ID, the object ID of the application, and the object ID of the service principal, in the following format.

        ```sh
        $ pulumi import azuread:index/applicationFromTemplate:ApplicationFromTemplate example /applicationTemplates/00000000-0000-0000-0000-000000000000/instantiate/11111111-1111-1111-1111-111111111111/22222222-2222-2222-2222-222222222222
        ```

        :param str resource_name: The name of the resource.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] display_name: The display name for the application.
        :param pulumi.Input[str] template_id: Unique ID for a templated application in the Azure AD App Gallery, from which to create the application. Changing this forces a new resource to be created.
        """
        ...
    @overload
    def __init__(__self__,
                 resource_name: str,
                 args: ApplicationFromTemplateArgs,
                 opts: Optional[pulumi.ResourceOptions] = None):
        """
        Creates an application registration and associated service principal from a gallery template.

        > The Application resource can also be used to instantiate a gallery application, however unlike the `Application` resource, this resource does not attempt to manage any properties of the resulting application.

        ## API Permissions

        The following API permissions are required in order to use this resource.

        When authenticated with a service principal, this resource requires one of the following application roles: `Application.ReadWrite.OwnedBy` or `Application.ReadWrite.All`

        When authenticated with a user principal, this resource may require one of the following directory roles: `Application Administrator` or `Global Administrator`

        ## Example Usage

        ```python
        import pulumi
        import pulumi_azuread as azuread

        example = azuread.get_application_template(display_name="Marketo")
        example_application_from_template = azuread.ApplicationFromTemplate("example",
            display_name="Example Application",
            template_id=example.template_id)
        example_get_application = azuread.get_application_output(object_id=example_application_from_template.application_object_id)
        example_get_service_principal = azuread.get_service_principal_output(object_id=example_application_from_template.service_principal_object_id)
        ```

        ## Import

        Templated Applications can be imported using the template ID, the object ID of the application, and the object ID of the service principal, in the following format.

        ```sh
        $ pulumi import azuread:index/applicationFromTemplate:ApplicationFromTemplate example /applicationTemplates/00000000-0000-0000-0000-000000000000/instantiate/11111111-1111-1111-1111-111111111111/22222222-2222-2222-2222-222222222222
        ```

        :param str resource_name: The name of the resource.
        :param ApplicationFromTemplateArgs args: The arguments to use to populate this resource's properties.
        :param pulumi.ResourceOptions opts: Options for the resource.
        """
        ...
    def __init__(__self__, resource_name: str, *args, **kwargs):
        resource_args, opts = _utilities.get_resource_args_opts(ApplicationFromTemplateArgs, pulumi.ResourceOptions, *args, **kwargs)
        if resource_args is not None:
            __self__._internal_init(resource_name, opts, **resource_args.__dict__)
        else:
            __self__._internal_init(resource_name, *args, **kwargs)

    def _internal_init(__self__,
                 resource_name: str,
                 opts: Optional[pulumi.ResourceOptions] = None,
                 display_name: Optional[pulumi.Input[str]] = None,
                 template_id: Optional[pulumi.Input[str]] = None,
                 __props__=None):
        opts = pulumi.ResourceOptions.merge(_utilities.get_resource_opts_defaults(), opts)
        if not isinstance(opts, pulumi.ResourceOptions):
            raise TypeError('Expected resource options to be a ResourceOptions instance')
        if opts.id is None:
            if __props__ is not None:
                raise TypeError('__props__ is only valid when passed in combination with a valid opts.id to get an existing resource')
            __props__ = ApplicationFromTemplateArgs.__new__(ApplicationFromTemplateArgs)

            if display_name is None and not opts.urn:
                raise TypeError("Missing required property 'display_name'")
            __props__.__dict__["display_name"] = display_name
            if template_id is None and not opts.urn:
                raise TypeError("Missing required property 'template_id'")
            __props__.__dict__["template_id"] = template_id
            __props__.__dict__["application_id"] = None
            __props__.__dict__["application_object_id"] = None
            __props__.__dict__["service_principal_id"] = None
            __props__.__dict__["service_principal_object_id"] = None
        super(ApplicationFromTemplate, __self__).__init__(
            'azuread:index/applicationFromTemplate:ApplicationFromTemplate',
            resource_name,
            __props__,
            opts)

    @staticmethod
    def get(resource_name: str,
            id: pulumi.Input[str],
            opts: Optional[pulumi.ResourceOptions] = None,
            application_id: Optional[pulumi.Input[str]] = None,
            application_object_id: Optional[pulumi.Input[str]] = None,
            display_name: Optional[pulumi.Input[str]] = None,
            service_principal_id: Optional[pulumi.Input[str]] = None,
            service_principal_object_id: Optional[pulumi.Input[str]] = None,
            template_id: Optional[pulumi.Input[str]] = None) -> 'ApplicationFromTemplate':
        """
        Get an existing ApplicationFromTemplate resource's state with the given name, id, and optional extra
        properties used to qualify the lookup.

        :param str resource_name: The unique name of the resulting resource.
        :param pulumi.Input[str] id: The unique provider ID of the resource to lookup.
        :param pulumi.ResourceOptions opts: Options for the resource.
        :param pulumi.Input[str] application_id: The resource ID for the application.
        :param pulumi.Input[str] application_object_id: The object ID for the application.
        :param pulumi.Input[str] display_name: The display name for the application.
        :param pulumi.Input[str] service_principal_id: The resource ID for the service principal.
        :param pulumi.Input[str] service_principal_object_id: The object ID for the service principal.
        :param pulumi.Input[str] template_id: Unique ID for a templated application in the Azure AD App Gallery, from which to create the application. Changing this forces a new resource to be created.
        """
        opts = pulumi.ResourceOptions.merge(opts, pulumi.ResourceOptions(id=id))

        __props__ = _ApplicationFromTemplateState.__new__(_ApplicationFromTemplateState)

        __props__.__dict__["application_id"] = application_id
        __props__.__dict__["application_object_id"] = application_object_id
        __props__.__dict__["display_name"] = display_name
        __props__.__dict__["service_principal_id"] = service_principal_id
        __props__.__dict__["service_principal_object_id"] = service_principal_object_id
        __props__.__dict__["template_id"] = template_id
        return ApplicationFromTemplate(resource_name, opts=opts, __props__=__props__)

    @property
    @pulumi.getter(name="applicationId")
    def application_id(self) -> pulumi.Output[str]:
        """
        The resource ID for the application.
        """
        return pulumi.get(self, "application_id")

    @property
    @pulumi.getter(name="applicationObjectId")
    def application_object_id(self) -> pulumi.Output[str]:
        """
        The object ID for the application.
        """
        return pulumi.get(self, "application_object_id")

    @property
    @pulumi.getter(name="displayName")
    def display_name(self) -> pulumi.Output[str]:
        """
        The display name for the application.
        """
        return pulumi.get(self, "display_name")

    @property
    @pulumi.getter(name="servicePrincipalId")
    def service_principal_id(self) -> pulumi.Output[str]:
        """
        The resource ID for the service principal.
        """
        return pulumi.get(self, "service_principal_id")

    @property
    @pulumi.getter(name="servicePrincipalObjectId")
    def service_principal_object_id(self) -> pulumi.Output[str]:
        """
        The object ID for the service principal.
        """
        return pulumi.get(self, "service_principal_object_id")

    @property
    @pulumi.getter(name="templateId")
    def template_id(self) -> pulumi.Output[str]:
        """
        Unique ID for a templated application in the Azure AD App Gallery, from which to create the application. Changing this forces a new resource to be created.
        """
        return pulumi.get(self, "template_id")

